unit FormRgn;
{ *************************************************
  *            FormRgn library 02.08.99           *
  *           written by Grigoriev Anton          *
  *                                               *
  *                    Main unit                  *
  *   This unit must be installed for the whole   *
  *                 library to work               *
  ************************************************* }

{ $Define Unregistred}

 interface

  uses Windows,Messages,SysUtils,Classes,Graphics,Controls,Forms,Dialogs,DsgnIntf,Menus,RgnEdit,ExtCtrls,Math{$IfDef Unregistred},StopForm{$EndIf};

   type EOwnerMustBeForm=class(Exception);
        ECannotTranslate=class(Exception);
        EDuplicateComponent=class(Exception);
        EOwnerNotAssigned=class(Exception);
        EPaintAlreadyHooked=class(Exception);
        EExtRgnCrossLink=class(Exception);

        TSizeChangedEvent=procedure(Sender:TObject;var Region:string) of object;

        TFRCoordinator=class;

        TExternalRegion=class;

        TRegionComp=class(TComponent)
                     private
                      FRegion:string;
                      Rgn:HRgn;
                      Coordinator:TFRCoordinator;
                      FOnSizeChanged:TSizeChangedEvent;
                      procedure SetRegion(const Value:string);
                     protected
                      property Region:string read FRegion write SetRegion;
                      property OnSizeChanged:TSizeChangedEvent read FOnSizeChanged write FOnSizeChanged default nil;
                      procedure Loaded;override;
                      procedure SizeChanged;dynamic;
                      procedure RegionCreated;dynamic;abstract;
                      procedure ExtRgnMessage(ExtRgn:TExternalRegion;Param:Integer);
                      procedure Refresh;
                     public
                      constructor Create(AOwner:TComponent);override;
                    end;

        TFormRegion=class(TRegionComp)
                     private
                      FDrawOnDesigning:Boolean;
                      FClientDrag:Boolean;
                      procedure SetDrawOnDesigning(Value:Boolean);
                     protected
                      procedure RegionCreated;override;
                      procedure HatchRegion;
                     public
                      constructor Create(AOwner:TComponent);override;
                      destructor Destroy;override;
                     published
                      property ClientDrag:Boolean read FClientDrag write FClientDrag default False;
                      property DrawOnDesigning:Boolean read FDrawOnDesigning write SetDrawOnDesigning default True;
                      property OnSizeChanged;
                      property Region;
                    end;

        TCaptionRegion=class;

        TCaptionLabel=class(TCollectionItem)
                       private
                        FColors:array[0..1] of TColor;
                        FText:string;
                        FFont:TFont;
                        FCoords:array[0..2] of Integer;
                        FVisible:Boolean;
                        function GetDisplayName:string;override;
                        procedure SetColors(Index:Integer;Value:TColor);
                        procedure SetText(Value:string);
                        procedure SetFont(Value:TFont);
                        procedure SetCoords(Index:Integer;Value:Integer);
                        procedure SetVisible(Value:Boolean);
                        procedure FontChanged(Sender:TObject);
                       public
                        constructor Create(Collection:TCollection);override;
                        destructor Destroy;override;
                        procedure Assign(Source:TPersistent);override;
                        procedure Draw(DC:HDC;CIndex:Integer);
                       published
                        property ActiveColor:TColor index 1 read FColors[1] write SetColors default clCaptionText;
                        property Angle:Integer index 2 read FCoords[2] write SetCoords default 0;
                        property Font:TFont read FFont write SetFont;
                        property InactiveColor:TColor index 0 read FColors[0] write SetColors default clInactiveCaptionText;
                        property Left:Integer index 0 read FCoords[0] write SetCoords default 0;
                        property Text:string read FText write SetText;
                        property Top:Integer index 1 read FCoords[1] write SetCoords default 0;
                        property Visible:Boolean read FVisible write SetVisible default True;
                      end;

        TCaptionLabels=class(TCollection)
                        private
                         CaptionRegion:TCaptionRegion;
                         function GetItem(Index:Integer):TCaptionLabel;
                         procedure SetItem(Index:Integer;Value:TCaptionLabel);
                        protected
                         function GetOwner:TPersistent;override;
                         procedure Update(Item:TCollectionItem);override;
                        public
                         constructor Create(ACaptionRegion:TCaptionRegion);
                         function Add:TCaptionLabel;
                         procedure Draw(DC:HDC;CIndex:Integer);
                         property Items[Index:Integer]:TCaptionLabel read GetItem write SetItem;default;
                       end;

        TSysMenuPicture=class(TPersistent)
                         private
                          FGlyph:TBitmap;
                          Owner:TCaptionRegion;
                          FCoords:array[0..1] of Integer;
                          FFlags:array[0..1] of Boolean;
                          FSize:array[0..1] of Integer;
                          procedure SetGlyph(Value:TBitmap);
                          procedure SetCoords(Index:Integer;Value:Integer);
                          procedure SetFlags(Index:Integer;Value:Boolean);
                          procedure SetSize(Index:Integer;Value:Integer);
                          procedure Redraw;
                         public
                          constructor Create(AOwner:TCaptionRegion);
                          destructor Destroy;override;
                          procedure Assign(Source:TPersistent);override;
                          procedure Draw(DC:HDC);
                          function RealWidth:Integer;
                          function RealHeight:Integer;
                         published
                          property Glyph:TBitmap read FGlyph write SetGlyph;
                          property Height:Integer index 0 read FSize[0] write SetSize default -1;
                          property Left:Integer index 0 read FCoords[0] write SetCoords default 0;
                          property Top:Integer index 1 read FCoords[1] write SetCoords default 0;
                          property Transparent:Boolean index 0 read FFlags[0] write SetFlags default True;
                          property Visible:Boolean index 1 read FFlags[1] write SetFlags default False;
                          property Width:Integer index 1 read FSize[1] write SetSize default -1;
                        end;

        TCaptionRegion=class(TRegionComp)
                        private
                         FColors:array [0..1] of TColor;
                         FPopupMenu:TPopupMenu;
                         FSysMenuPicture:TSysMenuPicture;
                         ClickTime:Integer;
                         FLabels:TCaptionLabels;
                         procedure SetColor(Index:Integer;Value:TColor);
                         procedure SetSysMenuPicture(Value:TSysMenuPicture);
                         procedure SetLabels(Value:TCaptionLabels);
                        protected
                         procedure DrawCaption(DC:HDC);
                         procedure RegionCreated;override;
                         procedure Loaded;override;
                         procedure InvalidateCaption;
                         function PointArea(LParam:Integer):Integer;
                         procedure Popup(X,Y:Integer);
                         procedure DefaultItem;
                        public
                         constructor Create(AOwner:TComponent);override;
                         destructor Destroy;override;
                        published
                         property ActiveColor:TColor index 1 read FColors[1] write SetColor default clActiveCaption;
                         property InactiveColor:TColor index 0 read FColors[0] write SetColor default clInactiveCaption;
                         property Labels:TCaptionLabels read FLabels write SetLabels;
                         property PopupMenu:TPopupMenu read FPopupMenu write FPopupMenu default nil;
                         property SysMenuPicture:TSysMenuPicture read FSysMenuPicture write SetSysMenuPicture;
                         property OnSizeChanged;
                         property Region;
                       end;

        TTransparency=0..100;
        TTransparencyStyle=(tsConstant,tsHorGradient,tsVertGradient,tsCustom);
        TBackgroundStyle=(bsSimple,bsMosaic,bsCentered,bsStretched,bsCustom);

        TGetTransparencyEvent=procedure(Sender:TObject;X,Y:Integer;Rect:TRect;var Transparency:TTransparency) of object;
        TGetGlassColorEvent=procedure(Sender:TObject;X,Y:Integer;Rect:TRect;var Red,Green,Blue:Byte) of object;

        TGlassRegion=class(TRegionComp)
                      private
                       FGlyph:TBitmap;
                       FColor:TColor;
                       FTransparency,FAltTransparency:TTransparency;
                       FTranspStyle:TTransparencyStyle;
                       FBackStyle:TBackgroundStyle;
                       FOnGetTransparency:TGetTransparencyEvent;
                       FOnGetGlassColor:TGetGlassColorEvent;
                       Next:TGlassRegion;
                       AGlyph:TBitmap;
                       Rect:TRect;
                       FirstCall:Boolean;
                       function GetDelay:Cardinal;
                       procedure SetDelay(Value:Cardinal);
                       function GetDrawOnDesigning:Boolean;
                       procedure SetDrawOnDesigning(Value:Boolean);
                       procedure SetColor(Value:TColor);
                       procedure SetTransparency(Value:TTransparency);
                       procedure SetAltTransparency(Value:TTransparency);
                       procedure SetBackStyle(Value:TBackgroundStyle);
                       procedure SetGlyph(Value:TBitmap);
                       procedure SetTranspStyle(Value:TTransparencyStyle);
                       function ConstantTransparency(X,Y:Integer):TTransparency;
                       function HGTransparency(X,Y:Integer):TTransparency;
                       function VGTransparency(X,Y:Integer):TTransparency;
                       function CustomTransparency(X,Y:Integer):TTransparency;
                       procedure SimpleBack(X,Y:Integer;var CR,CG,CB:Byte);
                       procedure MosaicBack(X,Y:Integer;var CR,CG,CB:Byte);
                       procedure CenteredBack(X,Y:Integer;var CR,CG,CB:Byte);
                       procedure StretchedBack(X,Y:Integer;var CR,CG,CB:Byte);
                       procedure CustomBack(X,Y:Integer;var CR,CG,CB:Byte);
                      protected
                       procedure RegionCreated;override;
                       procedure DrawStainedGlass(Back,Base:TBitmap);
                       procedure DraftGlass(Back:TBitmap);
                      public
                       constructor Create(AOwner:TComponent);override;
                       destructor Destroy;override;
                      published
                       property AltTransparency:TTransparency read FAltTransparency write SetAltTransparency default 100;
                       property BackStyle:TBackgroundStyle read FBackStyle write SetBackStyle default bsSimple;
                       property Color:TColor read FColor write SetColor default clBlack;
                       property DelayTime:Cardinal read GetDelay write SetDelay default 300;
                       property DrawOnDesigning:Boolean read GetDrawOnDesigning write SetDrawOnDesigning default False;
                       property Glyph:TBitmap read FGlyph write SetGlyph;
                       property Transparency:TTransparency read FTransparency write SetTransparency default 50;
                       property TranspStyle:TTransparencyStyle read FTranspStyle write SetTranspStyle default tsConstant;
                       property OnSizeChanged;
                       property Region;
                       property OnGetGlassColor:TGetGlassColorEvent read FOnGetGlassColor write FOnGetGlassColor default nil;
                       property OnGetTransparency:TGetTransparencyEvent read FOnGetTransparency write FOnGetTransparency default nil;
                     end;

        TExternalRegion=class(TComponent)
                         private
                          Coordinator:TFRCoordinator;
                          Next:TExternalRegion;
                          First:Boolean;
                          FOwnRegion:Boolean;
                          Rgn:HRgn;
                          FOldName:TComponentName;
                         protected
                          function MakeRgn:HRgn;virtual;abstract;
                          procedure RgnChanged;dynamic;
                          procedure SetName(const Value:TComponentName);override;
                          function RgnSize:TSize;dynamic;
                         public
                          constructor Create(AOwner:TComponent);override;
                          destructor Destroy;override;
                          function GetRgn:HRgn;
                         published
                          property OwnRegion:Boolean read FOwnRegion write FOwnRegion default True;
                        end;

        TTextExtRgn=class(TExternalRegion)
                     private
                      FText:string;
                      FAngle:Integer;
                      FWeight:Integer;
                      FFont:TFont;
                      FTransparent:Boolean;
                      procedure SetText(const Value:string);
                      procedure SetAngle(Value:Integer);
                      procedure SetWeight(Value:Integer);
                      procedure SetFont(Value:TFont);
                      procedure SetTransparent(Value:Boolean);
                      procedure FontChanged(Sender:TObject);
                     protected
                      function MakeRgn:HRgn;override;
                     public
                      constructor Create(AOwner:TComponent);override;
                      destructor Destroy;override;
                     published
                      property Angle:Integer read FAngle write SetAngle default 0;
                      property Font:TFont read FFont write SetFont;
                      property Text:string read FText write SetText;
                      property Transparent:Boolean read FTransparent write SetTransparent default True;
                      property Weight:Integer read FWeight write SetWeight default -1;
                    end;

        TRegionAlignment=(raResult,raSource);

        TPictExtRgn=class(TExternalRegion)
                     private
                      FGlyph:TBitmap;
                      FTransColor:TColor;
                      FInverted:Boolean;
                      FAlign:array[0..1] of TRegionAlignment;
                      procedure SetGlyph(Value:TBitmap);
                      procedure SetTransColor(Value:TColor);
                      procedure SetInverted(Value:Boolean);
                      procedure SetAlign(Index:Integer;Value:TRegionAlignment);
                     protected
                      function MakeRgn:HRgn;override;
                      function BitmapToRgn(Bitmap:TBitmap):HRgn;
                      function RgnSize:TSize;override;
                     public
                      constructor Create(AOwner:TComponent);override;
                      destructor Destroy;override;
                     published
                      property Glyph:TBitmap read FGlyph write SetGlyph;
                      property HorAlign:TRegionAlignment index 0 read FAlign[0] write SetAlign default raResult;
                      property Inverted:Boolean read FInverted write SetInverted default False;
                      property TransColor:TColor read FTransColor write SetTransColor default clOlive;
                      property VertAlign:TRegionAlignment index 1 read FAlign[1] write SetAlign default raResult;
                    end;

        TMacrosExtRgn=class(TExternalRegion)
                       private
                        FRegion:string;
                        procedure SetRegion(Value:string);
                       protected
                        function MakeRgn:HRgn;override;
                        procedure ExtRgnMessage(ExtRgn:TExternalRegion;Param:Integer);
                       public
                        constructor Create(AOwner:TComponent);override;
                       published
                        property Region:string read FRegion write SetRegion;
                      end;

        TMenuHotSpot=class(TCustomControl)
                      private
                       FGlyph:TBitmap;
                       FTransparent:Boolean;
                       FColors:array[0..1] of TColor;
                       ClickTime:Integer;
                       Coordinator:TFRCoordinator;
                       Next:TMenuHotSpot;
                       procedure SetGlyph(Value:TBitmap);
                       procedure SetTransparent(Value:Boolean);
                       procedure SetColor(Index:Integer;Value:TColor);
                      protected
                       procedure CallMenu;
                       procedure MouseDown(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);override;
                       procedure DblClick;override;
                       procedure Paint;override;
                      public
                       constructor Create(AOwner:TComponent);override;
                       destructor Destroy;override;
                      published
                       property ActiveColor:TColor index 1 read FColors[1] write SetColor default clActiveCaption;
                       property Glyph:TBitmap read FGlyph write SetGlyph;
                       property InactiveColor:TColor index 0 read FColors[0] write SetColor default clInactiveCaption;
                       property PopupMenu;
                       property Transparent:Boolean read FTransparent write SetTransparent;
                       property Visible;
                       property ShowHint;
                       property ParentShowHint;
                     end;

        TSysButtonKind=(sbClose,sbMinimize,sbMaximize,sbHelp,sbGlyph,sbOwnerDraw,sbOwnerDrawFace);

        TSystemButton=class(TCustomControl)
                       private
                        FKind:TSysButtonKind;
                        FAutoSize:Boolean;
                        FOnDrawButton:TNotifyEvent;
                        FPressed:Boolean;
                        FPressLoop:Boolean;
                        FGlyph:TBitmap;
                        Next:TSystemButton;
                        Coordinator:TFRCoordinator;
                        procedure SetKind(Value:TSysButtonKind);
                        procedure SetAutoSize(Value:Boolean);
                        procedure SetGlyph(Value:TBitmap);
                        procedure CMEnabledChanged(var Message:TMessage);message CM_EnabledChanged;
                       protected
                        procedure Paint;override;
                        procedure MouseDown(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);override;
                        procedure MouseMove(Shift:TShiftState;X,Y:Integer);override;
                        procedure MouseUp(Button:TMouseButton;Shift:TShiftState;X,Y:Integer);override;
                        procedure Loaded;override;
                       public
                        property Pressed:Boolean read FPressed;
                        property Canvas;
                        constructor Create(AOwner:TComponent);override;
                        destructor Destroy;override;
                        procedure SetBounds(ALeft,ATop,AWidth,AHeight:Integer);override;
                       published
                        property AutoSize:Boolean read FAutoSize write SetAutoSize default True;
                        property Glyph:TBitmap read FGlyph write SetGlyph;
                        property Kind:TSysButtonKind read FKind write SetKind default sbClose;
                        property Enabled;
                        property Visible;
                        property ShowHint;
                        property ParentShowHint;
                        property OnClick;
                        property OnDrawButton:TNotifyEvent read FOnDrawButton write FOnDrawButton default nil;
                      end;

        TCaptionText=class(TGraphicControl)
                      private
                       FAngle:Integer;
                       FColors:array[0..1] of TColor;
                       FAutoSize:Boolean;
                       Next:TCaptionText;
                       Coordinator:TFRCoordinator;
                       procedure SetAngle(Value:Integer);
                       procedure SetAutoSize(Value:Boolean);
                       procedure SetColor(Index:Integer;Value:TColor);
                      protected
                       function CalcSize:TSize;
                       procedure SetSize;
                       procedure CMTextChanged(var Message:TMessage);message CM_TextChanged;
                       procedure CMFontChanged(var Message:TMessage);message CM_FontChanged;
                       procedure Paint;override;
                      public
                       constructor Create(AOwner:TComponent);override;
                       destructor Destroy;override;
                      published
                       property ActiveColor:TColor index 1 read FColors[1] write SetColor default clCaptionText;
                       property Angle:Integer read FAngle write SetAngle default 0;
                       property AutoSize:Boolean read FAutoSize write SetAutoSize default True;
                       property InactiveColor:TColor index 0 read FColors[0] write SetColor default clInactiveCaptionText;
                       property Caption;
                       property Visible;
                       property Font;
                     end;

        TFRManager=class(TComponent)
                    private
                     Coordinator:TFRCoordinator;
                     CallCount:Integer;
                    public
                     constructor Create(AOwner:TComponent);override;
                     destructor Destroy;override;
                     procedure Activate;dynamic;
                     procedure Deactivate;dynamic;
                     procedure StayOnTop(Stay:Boolean);
                   end;

        TFRCoordinator=class
                        private
                         Form:TForm;
                         RefCount:Integer;
                         FormRegion:TFormRegion;
                         CaptionRegion:TCaptionRegion;
                         Glass:TGlassRegion;
                         ExtRgn:TExternalRegion;
                         HotSpot:TMenuHotSpot;
                         SysBtn:TSystemButton;
                         CptText:TCaptionText;
                         FDelayTime:Cardinal;
                         FDrawOnDesigning:Boolean;
                         Moving:Boolean;
                         OldWndProc:TFarProc;
                         ScrImage:TBitmap;
                         NameList:TStringList;
                         NewWndProc:Pointer;
                         Next:TFRCoordinator;
                         procedure HookForm;
                         procedure UnhookForm;
                        protected
                         procedure DecRefCount;
                         procedure AssignFormRegion(FRgn:TFormRegion);
                         procedure RemoveFormRegion;
                         procedure AssignCaptionRegion(CRgn:TCaptionRegion);
                         procedure RemoveCaptionRegion;
                         procedure AddGlassRegion(GlassRegion:TGlassRegion);
                         procedure DeleteGlassRegion(GlassRegion:TGlassRegion);
                         procedure AddExtRgn(ExternalRegion:TExternalRegion);
                         procedure DeleteExtRgn(ExternalRegion:TExternalRegion);
                         procedure AddMenuHotSpot(MenuHotSpot:TMenuHotSpot);
                         procedure DeleteMenuHotSpot(MenuHotSpot:TMenuHotSpot);
                         procedure AddSystemButton(SystemButton:TSystemButton);
                         procedure DeleteSystemButton(SystemButton:TSystemButton);
                         procedure AddCaptionText(CaptionText:TCaptionText);
                         procedure DeleteCaptionText(CaptionText:TCaptionText);
                         procedure AssignManager;
                         procedure RemoveManager;
                         procedure ExtRgnMessage(ExtRgn:TExternalRegion;Param:Integer);
                         function CreateRegion(Region:string;SelfName:string):HRgn;
                         function GetXOrigin:Integer;
                         function GetYOrigin:Integer;
                         function DesignWidth:Integer;
                         function DesignHeight:Integer;
                         procedure CallDefault(var Message:TMessage);
                         procedure HookWndProc(var Message:TMessage);
                         procedure Activate;
                         procedure Deactivate;
                        public
                         constructor Create(AForm:TForm);
                         destructor Destroy;override;
                       end;

    procedure Register;

 implementation

  type PPointsArray=^TPointsArray;
       TPointsArray=array[1..1000000] of TPoint;

       PRGBArray=^TRGBArray;
       TRGBArray=array[0..1000000] of TRGBTriple;

       TTranspFunc=function(X,Y:Integer):TTransparency of object;
       TColorProc=procedure(X,Y:Integer;var CR,CG,CB:Byte) of object;

   var DFrame,SFrame,SzFrame:TPoint;
       NCapt,SmCapt:Integer;
       CHead:TFRCoordinator;

    function FindCoordinator(Form:TForm):TFRCoordinator;
     var P:TFRCoordinator;
      begin
       P:=CHead;
       while Assigned(P) do
        begin
         if P.Form=Form then
          begin
           Result:=P;
           Exit
          end;
         P:=P.Next
        end;
       Result:=TFRCoordinator.Create(Form)
      end;

    procedure CallDefaultItem(Menu:TMenu);
     var I:Integer;
      begin
       for I:=0 to Menu.Items.Count-1 do
        if Menu.Items[I].Default then
         begin
          Menu.Items[I].Click;
          Break
         end
      end;

    {TRegionComp}

    constructor TRegionComp.Create;
     begin
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of '+ClassName+' component must be the TForm descendant');
      inherited Create(AOwner);
      Coordinator:=FindCoordinator(TForm(Owner));
      FRegion:='';
      Rgn:=0
     end;

    procedure TRegionComp.Loaded;
     begin
      inherited Loaded;
      Region:=Region
     end;

    procedure TRegionComp.SizeChanged;
     var NewRegion:string;
      begin
       with TForm(Owner) do
        begin
         NewRegion:=Region;
         if Assigned(FOnSizeChanged) then
          FOnSizeChanged(Self,NewRegion);
         Region:=NewRegion
        end
      end;

    procedure TRegionComp.SetRegion;
     begin
      FRegion:=Value;
      if not (csLoading in ComponentState) then
       begin
        if Rgn<>0 then
         DeleteObject(Rgn);
        Rgn:=Coordinator.CreateRegion(Region,Name);
        RegionCreated
       end
     end;

    procedure TRegionComp.ExtRgnMessage;
     var OldName,NewRgn:string;
         P:Integer;
      begin
       case Param of
        1:if Pos(UpperCase(ExtRgn.Name),UpperCase(FRegion))<>0 then
           Region:=Region;
        2:begin
           OldName:=UpperCase(ExtRgn.FOldName);
           if OldName<>'' then
            begin
             NewRgn:=FRegion;
             P:=Pos(OldName,UpperCase(NewRgn));
             while P<>0 do
              begin
               Delete(NewRgn,P,Length(OldName));
               Insert(ExtRgn.Name,NewRgn,P);
               P:=Pos(OldName,UpperCase(NewRgn))
              end;
             Region:=NewRgn
            end
          end
       end
      end;

    procedure TRegionComp.Refresh;
     begin
      Region:=Region
     end;

    {TFormRegion}

    constructor TFormRegion.Create;
     var I:Integer;
      begin
       with AOwner do
        for I:=0 to ComponentCount-1 do
         if (Components[I] is TFormRegion) and (Components[I]<>Self) then
          raise EDuplicateComponent.Create('Only one TFormRegion component on a form is allowed');
       inherited Create(AOwner);
       FDrawOnDesigning:=True;
       FClientDrag:=False;
       Coordinator.AssignFormRegion(Self)
      end;

    destructor TFormRegion.Destroy;
     var R:HRgn;
      begin
       if Assigned(Owner) and (csDesigning in ComponentState) and DrawOnDesigning and not (csDestroying in Owner.ComponentState) then
        with TForm(Owner) do
         begin
          Invalidate;
          R:=CreateRectRgnIndirect(BoundsRect);
          SendMessage(Handle,WM_NCPaint,R,0);
          DeleteObject(R)
         end;
       Coordinator.RemoveFormRegion;
       inherited Destroy
      end;

    procedure TFormRegion.HatchRegion;
     var DC:HDC;
         R2:HRgn;
         MBrush:HBrush;
      begin
       if (csDesigning in ComponentState) and (Rgn<>0) and DrawOnDesigning then
        with TForm(Owner) do
         begin
          DC:=GetWindowDC(Handle);
          SetBkMode(DC,Transparent);
          R2:=CreateRectRgn(0,0,Width,Height);
          CombineRgn(R2,R2,Rgn,Rgn_Diff);
          OffsetRgn(R2,Coordinator.GetXOrigin,Coordinator.GetYOrigin);
          MBrush:=CreateHatchBrush(hs_DiagCross,clBlack);
          FillRgn(DC,R2,MBrush);
          DeleteObject(MBrush);
          DeleteObject(R2);
          ReleaseDC(Handle,DC)
         end
      end;

    procedure TFormRegion.SetDrawOnDesigning;
     var R:HRgn;
      begin
       if Value<>FDrawOnDesigning then
        begin
         FDrawOnDesigning:=Value;
         if (csDesigning in ComponentState) and not (csDestroying in Owner.ComponentState) then
          begin
           TForm(Owner).Invalidate;
           if not Value then
            with TForm(Owner) do
             begin
              R:=CreateRectRgnIndirect(BoundsRect);
              SendMessage(Handle,WM_NCPaint,R,0);
              DeleteObject(R)
             end;
          end
        end
      end;

    procedure TFormRegion.RegionCreated;
     var R2:HRgn;
      begin
       if not (csDesigning in ComponentState) and Assigned(Owner) and (Rgn<>0) then
        begin
         R2:=CreateRectRgn(0,0,1,1);
         CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
         SetWindowRgn((Owner as TForm).Handle,R2,(Owner as TForm).Visible)
        end
       else
        if DrawOnDesigning then
         (Owner as TForm).Invalidate
      end;

    {TCaptionLabel}

    constructor TCaptionLabel.Create;
     begin
      inherited Create(Collection);
      FCoords[0]:=0;
      FCoords[1]:=0;
      FCoords[2]:=0;
      FColors[0]:=clInactiveCaptionText;
      FColors[1]:=clCaptionText;
      FVisible:=True;
      FText:='';
      FFont:=TFont.Create;
      FFont.OnChange:=FontChanged
     end;

    destructor TCaptionLabel.Destroy;
     begin
      FFont.Free;
      inherited Destroy
     end;

    function TCaptionLabel.GetDisplayName;
     begin
      if Text='' then
       Result:=inherited GetDisplayName
      else
       Result:=Text
     end;

    procedure TCaptionLabel.Assign;
     begin
      if Source is TCaptionLabel then
       with TCaptionLabel(Source) do
        begin
         Self.FColors:=FColors;
         Self.Text:=Text;
         Self.Font:=Font;
         Self.FCoords:=FCoords;
         Self.Visible:=Visible;
        end
      else
       inherited Assign(Source)
     end;

    procedure TCaptionLabel.FontChanged;
     begin
      Changed(False)
     end;

    procedure TCaptionLabel.SetColors;
     begin
      if FColors[Index]<>Value then
       begin
        FColors[Index]:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.SetText;
     begin
      if FText<>Value then
       begin
        FText:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.SetFont;
     begin
      FFont.Assign(Value);
      Changed(False)
     end;

    procedure TCaptionLabel.SetCoords;
     begin
      if FCoords[Index]<>Value then
       begin
        FCoords[Index]:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.SetVisible;
     begin
      if FVisible<>Value then
       begin
        FVisible:=Value;
        Changed(False)
       end
     end;

    procedure TCaptionLabel.Draw;
     var LF:TLogFont;
         NewFont,OldFont:HFont;
      begin
       with LF,Font do
        begin
         lfHeight:=Height;
         lfWidth:=0;
         lfEscapement:=Angle;
         lfOrientation:=Angle;
         if fsBold in Style then
          lfWeight:=FW_Bold
         else
          lfWeight:=FW_Normal;
         lfItalic:=Byte(fsItalic in Style);
         lfUnderline:=Byte(fsUnderline in Style);
         lfStrikeOut:=Byte(fsStrikeOut in Style);
         lfCharSet:=Byte(CharSet);
         lfOutPrecision:=Out_Default_Precis;
         lfClipPrecision:=Clip_Default_Precis;
         lfQuality:=Default_Quality;
         case Pitch of
          fpVariable:lfPitchAndFamily:=Variable_Pitch;
          fpFixed:lfPitchAndFamily:=Fixed_Pitch
         else
          lfPitchAndFamily:=Default_Pitch
         end;
         StrPCopy(lfFaceName,Name)
        end;
       SetBkMode(DC,Transparent);
       NewFont:=CreateFontIndirect(LF);
       OldFont:=SelectObject(DC,NewFont);
       SetTextColor(DC,ColorToRGB(FColors[CIndex]));
       TextOut(DC,Left,Top,PChar(FText),Length(FText));
       SelectObject(DC,OldFont);
       DeleteObject(NewFont)
      end;

    {TCaptionLabels}

    constructor TCaptionLabels.Create;
     begin
      inherited Create(TCaptionLabel);
      CaptionRegion:=ACaptionRegion
     end;

    function TCaptionLabels.Add;
     begin
      Result:=TCaptionLabel(inherited Add)
     end;

    function TCaptionLabels.GetItem;
     begin
      Result:=TCaptionLabel(inherited GetItem(Index))
     end;

    procedure TCaptionLabels.SetItem;
     begin
      inherited SetItem(Index,Value)
     end;

    function TCaptionLabels.GetOwner;
     begin
      Result:=CaptionRegion
     end;

    procedure TCaptionLabels.Update;
     begin
      TForm(CaptionRegion.Owner).Invalidate
     end;

    procedure TCaptionLabels.Draw;
     var I:Integer;
      begin
       for I:=0 to Count-1 do
        if Items[I].Visible then
         Items[I].Draw(DC,CIndex)
      end;

    {TSysMenuPicture}

    constructor TSysMenuPicture.Create;
     begin
      if not Assigned(AOwner) then
       raise EOwnerNotAssigned.Create('TSysMenuPicture must have a owner');
      inherited Create;
      Owner:=AOwner;
      FGlyph:=TBitmap.Create;
      FSize[0]:=-1;
      FSize[1]:=-1;
      FCoords[0]:=0;
      FCoords[1]:=0;
      FFlags[0]:=True;
      FFlags[1]:=False
     end;

    destructor TSysMenuPicture.Destroy;
     begin
      FGlyph.Free;
      inherited Destroy
     end;

    procedure TSysMenuPicture.Assign;
     begin
      if Source is TSysMenuPicture then
       with TSysMenuPicture(Source) do
        begin
         Self.FGlyph.Assign(FGlyph);
         Self.FCoords:=FCoords;
         Self.FFlags:=FFlags;
         Self.FSize:=FSize
        end;
      inherited Assign(Source)
     end;

    procedure TSysMenuPicture.SetGlyph;
     begin
      FGlyph.Assign(Value);
      FGlyph.TransparentMode:=tmAuto;
      FGlyph.Transparent:=Transparent;
      Redraw
     end;

    procedure TSysMenuPicture.SetCoords;
     begin
      if Value<>FCoords[Index] then
       begin
        FCoords[Index]:=Value;
        Redraw
       end
     end;

    procedure TSysMenuPicture.SetFlags;
     begin
      if Value<>FFlags[Index] then
       begin
        FFlags[Index]:=Value;
        if Index=0 then
         begin
          FGlyph.TransparentMode:=tmAuto;
          FGlyph.Transparent:=Value;
          Redraw
         end
        else
         TForm(Owner.Owner).Invalidate
       end
     end;

    procedure TSysMenuPicture.SetSize;
     begin
      if Value<>FSize[Index] then
       begin
        FSize[Index]:=Value;
        if FGlyph.Empty then
         Redraw
       end
     end;

    procedure TSysMenuPicture.Redraw;
     begin
      if Visible then
       TForm(Owner.Owner).Invalidate
     end;

    procedure TSysMenuPicture.Draw;
     var Canvas:TCanvas;
         Icon:HIcon;
         W,H,DX,DY:Integer;
         R:TRect;
      begin
       if Visible then
        begin
         DX:=Left;
         DY:=Top;
         if not FGlyph.Empty then
          begin
           Canvas:=TCanvas.Create;
           Canvas.Handle:=DC;
           Canvas.Draw(DX,DY,FGlyph);
           Canvas.Handle:=0;
           Canvas.Free
          end
         else
          begin
           W:=Width;
           H:=Height;
           if W<=0 then
            W:=GetSystemMetrics(SM_CXSmIcon);
           if H<=0 then
            H:=GetSystemMetrics(SM_CYSmIcon);
           if csDesigning in Owner.ComponentState then
            begin
             R:=Rect(0,0,W,H);
             with TCaptionRegion(Owner).Coordinator do
              OffsetRect(R,DX,DY);
             SelectObject(DC,GetStockObject(White_Brush));
             Rectangle(DC,R.Left,R.Top,R.Right,R.Bottom)
            end
           else
            begin
             Icon:=LoadImage(HInstance,'MAINICON',Image_Icon,W,H,0);
             DrawIconEx(DC,DX,DY,Icon,W,H,0,0,DI_Normal);
             DeleteObject(Icon)
            end
          end
        end
      end;

    function TSysMenuPicture.RealWidth;
     begin
      if FGlyph.Empty then
       if Width>0 then
        Result:=Width
       else
        Result:=GetSystemMetrics(SM_CXSmIcon)
      else
       Result:=FGlyph.Width
     end;

    function TSysMenuPicture.RealHeight;
     begin
      if FGlyph.Empty then
       if Height>0 then
        Result:=Height
       else
        Result:=GetSystemMetrics(SM_CYSmIcon)
      else
       Result:=FGlyph.Height
     end;

    {TCaptionRegion}

    constructor TCaptionRegion.Create;
     var I:Integer;
      begin
       with TForm(AOwner) do
        for I:=0 to ComponentCount-1 do
         if (Components[I] is TCaptionRegion) and (Components[I]<>Self) then
          raise EDuplicateComponent.Create('Only one TCaptionRegion component on a form is allowed');
       inherited Create(AOwner);
       FColors[1]:=clActiveCaption;
       FColors[0]:=clInactiveCaption;
       FOnSizeChanged:=nil;
       FSysMenuPicture:=TSysMenuPicture.Create(Self);
       FLabels:=TCaptionLabels.Create(Self);
       ClickTime:=0;
       Coordinator.AssignCaptionRegion(Self)
      end;

    destructor TCaptionRegion.Destroy;
     var R:HRgn;
      begin
       if Assigned(Owner) and not (csDestroying in Owner.ComponentState) then
        with TForm(Owner) do
         begin
          Invalidate;
          R:=CreateRectRgnIndirect(BoundsRect);
          SendMessage(Handle,WM_NCPaint,R,0);
          DeleteObject(R)
         end;
       FLabels.Free;
       FSysMenuPicture.Free;
       Coordinator.RemoveCaptionRegion;
       inherited Destroy
      end;

    procedure TCaptionRegion.RegionCreated;
     begin
      TForm(Owner).Invalidate
     end;

    procedure TCaptionRegion.Loaded;
     begin
      inherited Loaded;
      with FSysMenuPicture.FGlyph do
       if not Empty then
        begin
         TransparentMode:=tmAuto;
         Transparent:=FSysMenuPicture.Transparent
        end
     end;

    procedure TCaptionRegion.SetColor;
     begin
      if FColors[Index]<>Value then
       begin
        FColors[Index]:=Value;
        TForm(Owner).Invalidate
       end
     end;

    procedure TCaptionRegion.SetSysMenuPicture;
     begin
      FSysMenuPicture.Assign(Value);
      TForm(Owner).Invalidate
     end;

    procedure TCaptionRegion.SetLabels;
     begin
      FLabels.Assign(Value);
      TForm(Owner).Invalidate
     end;

    procedure TCaptionRegion.DrawCaption;
     var R2:HRgn;
         N:Integer;
         MBrush:HBrush;
      begin
       with Coordinator do
        begin
         N:=Integer(Form.Active);
         SetBkMode(DC,Transparent);
         if (Rgn<>0) and (FColors[N]<>clNone) then
          begin
           R2:=CreateRectRgn(0,0,1,1);
           CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
           OffsetRgn(R2,Form.Left-Form.ClientOrigin.X+GetXOrigin,Form.Top-Form.ClientOrigin.Y+GetYOrigin);
           MBrush:=CreateSolidBrush(ColorToRGB(FColors[N]));
           FillRgn(DC,R2,MBrush);
           DeleteObject(MBrush);
           DeleteObject(R2)
          end;
         if FSysMenuPicture.Visible then
          FSysMenuPicture.Draw(DC);
         FLabels.Draw(DC,N);
        end
      end;

    procedure TCaptionRegion.InvalidateCaption;
     var R2:HRgn;
         N:Integer;
      begin
       N:=Integer(TForm(Owner).Active);
       if (Rgn<>0) and (FColors[N]<>clNone) then
        begin
         R2:=CreateRectRgn(0,0,1,1);
         CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
         with TForm(Owner) do
          begin
           OffsetRgn(R2,Left-ClientOrigin.X+Coordinator.GetXOrigin,Top-ClientOrigin.Y+Coordinator.GetYOrigin);
           InvalidateRgn(Handle,R2,True)
          end;
         DeleteObject(R2)
        end
      end;

    function TCaptionRegion.PointArea;
     var Pt:TPoint;
         R:TRect;
      begin
       Pt:=SmallPointToPoint(TSmallPoint(LParam));
       with FSysMenuPicture do
        R:=Rect(0,0,RealWidth,RealHeight);
       with TForm(Owner) do
        OffsetRect(R,FSysMenuPicture.Left+ClientOrigin.X,FSysMenuPicture.Top+ClientOrigin.Y);
       if FSysMenuPicture.Visible and PtInRect(R,Pt) then
        Result:=2
       else
        if PtInRegion(Rgn,Pt.X-TForm(Owner).Left,Pt.Y-TForm(Owner).Top) then
         Result:=1
        else
         Result:=0
      end;

    procedure TCaptionRegion.Popup;
     begin
      if Assigned(FPopupMenu) then
       if X=-1 then
        with TForm(Owner) do
         FPopupMenu.Popup(FSysMenuPicture.Left+ClientOrigin.X,FSysMenuPicture.Top+FSysMenuPicture.RealHeight+ClientOrigin.Y)
       else
        FPopupMenu.Popup(X,Y)
     end;

    procedure TCaptionRegion.DefaultItem;
     begin
      if Assigned(FPopupMenu) then
       CallDefaultItem(FPopupMenu)
     end;

    {TGlassRegion}

    constructor TGlassRegion.Create;
     begin
      inherited Create(AOwner);
      FColor:=clBlack;
      FTransparency:=50;
      Next:=nil;
      FGlyph:=TBitmap.Create;
      FAltTransparency:=100;
      FBackStyle:=bsSimple;
      FTranspStyle:=tsConstant;
      FOnGetGlassColor:=nil;
      FOnGetTransparency:=nil;
      Coordinator.AddGlassRegion(Self)
     end;

    destructor TGlassRegion.Destroy;
     begin
      FGlyph.Free;
      Coordinator.DeleteGlassRegion(Self);
      if Assigned(Owner) and not (csDestroying in Owner.ComponentState) then
       TForm(Owner).Invalidate;
      inherited Destroy
     end;

    procedure TGlassRegion.RegionCreated;
     begin
      TForm(Owner).Invalidate
     end;

    function TGlassRegion.GetDelay;
     begin
      Result:=Coordinator.FDelayTime
     end;

    procedure TGlassRegion.SetDelay;
     begin
      Coordinator.FDelayTime:=Value
     end;

    function TGlassRegion.GetDrawOnDesigning;
     begin
      Result:=Coordinator.FDrawOnDesigning
     end;

    procedure TGlassRegion.SetDrawOnDesigning;
     begin
      if Coordinator.FDrawOnDesigning<>Value then
       begin
        Coordinator.FDrawOnDesigning:=Value;
        if csDesigning in ComponentState then
         TForm(Owner).Invalidate
       end
     end;

    procedure TGlassRegion.SetColor;
     begin
      if FColor<>Value then
       begin
        FColor:=Value;
        TForm(Owner).Invalidate
       end
     end;

    procedure TGlassRegion.SetTransparency;
     begin
      if FTransparency<>Value then
       begin
        FTransparency:=Value;
        TForm(Owner).Invalidate
       end
     end;

    procedure TGlassRegion.SetAltTransparency;
     begin
      if Value<>FAltTransparency then
       begin
        FAltTransparency:=Value;
        if FTranspStyle<>tsConstant then
         TForm(Owner).Invalidate
       end
     end;

    procedure TGlassRegion.SetBackStyle;
     begin
      if Value<>FBackStyle then
       begin
        FBackStyle:=Value;
        TForm(Owner).Invalidate
       end
     end;

    procedure TGlassRegion.SetGlyph;
     begin
      FGlyph.Assign(Value);
      TForm(Owner).Invalidate
     end;

    procedure TGlassRegion.SetTranspStyle;
     begin
      if Value<>FTranspStyle then
       begin
        FTranspStyle:=Value;
        TForm(Owner).Invalidate
       end
     end;

    function TGlassRegion.ConstantTransparency;
     begin
      Result:=FTransparency
     end;

    function TGlassRegion.HGTransparency;
     begin
      Result:=FTransparency+Round((FAltTransparency-FTransparency)/(Rect.Right-Rect.Left)*X)
     end;

    function TGlassRegion.VGTransparency;
     begin
      Result:=FTransparency+Round((FAltTransparency-FTransparency)/(Rect.Bottom-Rect.Top)*Y)
     end;

    function TGlassRegion.CustomTransparency;
     begin
      FOnGetTransparency(Self,X,Y,Rect,Result)
     end;

    procedure TGlassRegion.SimpleBack;
     begin
      CR:=GetRValue(ColorToRGB(FColor));
      CG:=GetGValue(ColorToRGB(FColor));
      CB:=GetBValue(ColorToRGB(FColor))
     end;

    procedure TGlassRegion.MosaicBack;
     const OldY:Integer=-1;
           SL:PRGBArray=nil;
      var RX:Integer;
       begin
        if Y<>OldY then
         begin
          OldY:=Y;
          SL:=AGlyph.ScanLine[Y mod AGlyph.Height]
         end;
        RX:=X mod AGlyph.Width;
        CR:=SL[RX].rgbtRed;
        CG:=SL[RX].rgbtGreen;
        CB:=SL[RX].rgbtBlue
       end;

    procedure TGlassRegion.CenteredBack;
     const OldY:Integer=-1;
           SL:PRGBArray=nil;
           X1:Integer=0;
           Y1:Integer=0;
           X2:Integer=0;
           Y2:Integer=0;
      var RX:Integer;
       begin
        if FirstCall then
         begin
          FirstCall:=False;
          X1:=(Rect.Right-Rect.Left-AGlyph.Width) div 2;
          X2:=X1+AGlyph.Width;
          Y1:=(Rect.Bottom-Rect.Top-AGlyph.Height) div 2;
          Y2:=Y1+AGlyph.Height
         end;
        if Y<>OldY then
         begin
          OldY:=Y;
          if (Y>=Y1) and (Y<Y2) then
           SL:=AGlyph.ScanLine[Y-Y1]
          else
           SL:=nil
         end;
        if Assigned(SL) and (X>=X1) and (X<X2) then
         begin
          RX:=X-X1;
          CR:=SL[RX].rgbtRed;
          CG:=SL[RX].rgbtGreen;
          CB:=SL[RX].rgbtBlue
         end
        else
         begin
          CR:=GetRValue(ColorToRGB(FColor));
          CG:=GetGValue(ColorToRGB(FColor));
          CB:=GetBValue(ColorToRGB(FColor))
         end
       end;

    procedure TGlassRegion.StretchedBack;
     const OldY:Integer=-1;
           SL:PRGBArray=nil;
      var RX:Integer;
       begin
        if Y<>OldY then
         begin
          OldY:=Y;
          SL:=AGlyph.ScanLine[Round(Y/(Rect.Bottom-Rect.Top-1)*(AGlyph.Height-1))]
         end;
        RX:=Round(X/(Rect.Right-Rect.Left-1)*(AGlyph.Width-1));
        CR:=SL[RX].rgbtRed;
        CG:=SL[RX].rgbtGreen;
        CB:=SL[RX].rgbtBlue
       end;

    procedure TGlassRegion.CustomBack;
     begin
      FOnGetGlassColor(Self,X,Y,Rect,CR,CG,CB)
     end;

    procedure TGlassRegion.DrawStainedGlass;
     var R2:HRgn;
         R:TRect;
         X,Y,RX,RY,RX2:Integer;
         SL,BL:PRGBArray;
         CR,CG,CB:Byte;
         RT,ST:TTransparency;
         B2:TBitmap;
         TFunc:TTranspFunc;
         CProc:TColorProc;
      begin
       if Rgn<>0 then
        begin
         R2:=CreateRectRgn(0,0,1,1);
         CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
         with Coordinator do
          OffsetRgn(R2,Form.Left-Form.ClientOrigin.X+GetXOrigin,Form.Top-Form.ClientOrigin.Y+GetYOrigin);
         GetRgnBox(R2,R);
         if (R.Left<TForm(Owner).ClientWidth) and (R.Right>0) and (R.Bottom>0) and (R.Top<TForm(Owner).ClientHeight) then
          begin
           Rect:=R;
           if R.Left<0 then
            R.Left:=0;
           if R.Top<0 then
            R.Top:=0;
           if R.Right>TForm(Owner).ClientWidth then
            R.Right:=TForm(Owner).ClientWidth;
           if R.Bottom>TForm(Owner).ClientHeight then
            R.Bottom:=TForm(Owner).ClientHeight;
           B2:=TBitmap.Create;
           B2.Width:=R.Right-R.Left;
           B2.Height:=R.Bottom-R.Top;
           B2.PixelFormat:=pf24bit;
           case FTranspStyle of
            tsConstant:TFunc:=ConstantTransparency;
            tsHorGradient:TFunc:=HGTransparency;
            tsVertGradient:TFunc:=VGTransparency;
            tsCustom:if Assigned(FOnGetTransparency) then
                      TFunc:=CustomTransparency
                     else
                      TFunc:=ConstantTransparency
           end;
           case FBackStyle of
            bsSimple:CProc:=SimpleBack;
            bsMosaic:if FGlyph.Empty then
                      CProc:=SimpleBack
                     else
                      CProc:=MosaicBack;
            bsCentered:if FGlyph.Empty then
                        CProc:=SimpleBack
                       else
                        CProc:=CenteredBack;
            bsStretched:if FGlyph.Empty then
                         CProc:=SimpleBack
                        else
                         CProc:=StretchedBack;
            bsCustom:if Assigned(FOnGetGlassColor) then
                      CProc:=CustomBack
                     else
                      CProc:=SimpleBack
           end;
           if (FBackStyle in [bsMosaic..bsStretched]) and not FGlyph.Empty then
            begin
             AGlyph:=TBitmap.Create;
             AGlyph.Width:=FGlyph.Width;
             AGlyph.Height:=FGlyph.Height;
             AGlyph.PixelFormat:=pf24bit;
             AGlyph.Canvas.Draw(0,0,FGlyph)
            end
           else
            AGlyph:=nil;
           FirstCall:=True;
           for Y:=R.Top to R.Bottom-1 do
            begin
             RY:=Y-Rect.Top;
             SL:=Back.ScanLine[Y];
             BL:=B2.ScanLine[Y-R.Top];
             for X:=R.Left to R.Right-1 do
              begin
               RX:=X-Rect.Left;
               RX2:=X-R.Left;
               RT:=TFunc(RX,RY);
               ST:=100-RT;
               CProc(RX,RY,CR,CG,CB);
               BL[RX2].rgbtRed:=(RT*SL[X].rgbtRed+ST*CR) div 100;
               BL[RX2].rgbtGreen:=(RT*SL[X].rgbtGreen+ST*CG) div 100;
               BL[RX2].rgbtBlue:=(RT*SL[X].rgbtBlue+ST*CB) div 100
              end
            end;
           AGlyph.Free;
           SelectClipRgn(Base.Canvas.Handle,R2);
           Base.Canvas.Draw(R.Left,R.Top,B2);
           SelectClipRgn(Base.Canvas.Handle,0);
           SelectClipRgn(Back.Canvas.Handle,R2);
           Back.Canvas.Draw(R.Left,R.Top,B2);
           SelectClipRgn(Back.Canvas.Handle,0);
           B2.Free
          end;
         DeleteObject(R2)
        end
      end;

    procedure TGlassRegion.DraftGlass;
     var R2:HRgn;
         X,Y,RX2:Integer;
         B2:TBitmap;
         CProc:TColorProc;
         R:TRect;
         SL:PRGBArray;
         CR,CG,CB:Byte;
      begin
       if Rgn<>0 then
        begin
         R2:=CreateRectRgn(0,0,1,1);
         CombineRgn(R2,Rgn,Rgn,Rgn_Copy);
         with Coordinator do
          OffsetRgn(R2,Form.Left-Form.ClientOrigin.X+GetXOrigin,Form.Top-Form.ClientOrigin.Y+GetYOrigin);
         GetRgnBox(R2,R);
         if (R.Left<TForm(Owner).ClientWidth) and (R.Right>0) and (R.Bottom>0) and (R.Top<TForm(Owner).ClientHeight) then
          begin
           Rect:=R;
           if R.Left<0 then
            R.Left:=0;
           if R.Top<0 then
            R.Top:=0;
           if R.Right>TForm(Owner).ClientWidth then
            R.Right:=TForm(Owner).ClientWidth;
           if R.Bottom>TForm(Owner).ClientHeight then
            R.Bottom:=TForm(Owner).ClientHeight;
           B2:=TBitmap.Create;
           B2.Width:=R.Right-R.Left;
           B2.Height:=R.Bottom-R.Top;
           B2.PixelFormat:=pf24bit;
           B2.Canvas.Brush.Color:=TForm(Owner).Color;
           B2.Canvas.FillRect(Classes.Rect(0,0,B2.Width,B2.Height));
           case FBackStyle of
            bsSimple:CProc:=SimpleBack;
            bsMosaic:if FGlyph.Empty then
                      CProc:=SimpleBack
                     else
                      CProc:=MosaicBack;
            bsCentered:if FGlyph.Empty then
                        CProc:=SimpleBack
                       else
                        CProc:=CenteredBack;
            bsStretched:if FGlyph.Empty then
                         CProc:=SimpleBack
                        else
                         CProc:=StretchedBack;
            bsCustom:if Assigned(FOnGetGlassColor) then
                      CProc:=CustomBack
                     else
                      CProc:=SimpleBack
           end;
           if (FBackStyle in [bsMosaic..bsStretched]) and not FGlyph.Empty then
            begin
             AGlyph:=TBitmap.Create;
             AGlyph.Width:=FGlyph.Width;
             AGlyph.Height:=FGlyph.Height;
             AGlyph.PixelFormat:=pf24bit;
             AGlyph.Canvas.Draw(0,0,FGlyph)
            end
           else
            AGlyph:=nil;
           FirstCall:=True;
           for Y:=R.Top to R.Bottom-1 do
            begin
             SL:=B2.ScanLine[Y-R.Top];
             for X:=R.Left to R.Right-1 do
              if Odd(X) xor Odd(Y) then
               begin
                RX2:=X-R.Left;
                CProc(X-Rect.Left,Y-Rect.Top,CR,CG,CB);
                SL[RX2].rgbtRed:=CR;
                SL[RX2].rgbtGreen:=CG;
                SL[RX2].rgbtBlue:=CB
               end
             end;
           AGlyph.Free;
           SelectClipRgn(Back.Canvas.Handle,R2);
           Back.Canvas.Draw(R.Left,R.Top,B2);
           SelectClipRgn(Back.Canvas.Handle,0);
           B2.Free
          end;
         DeleteObject(R2)
        end
      end;

    {TExternalRegion}

    constructor TExternalRegion.Create;
     begin
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of '+ClassName+' component must be the TForm descendant');
      inherited Create(AOwner);
      Coordinator:=FindCoordinator(TForm(Owner));
      First:=True;
      Next:=nil;
      FOwnRegion:=True;
      Rgn:=0;
      FOldName:='';
      Coordinator.AddExtRgn(Self)
     end;

    destructor TExternalRegion.Destroy;
     begin
      if Rgn<>0 then
       DeleteObject(Rgn);
      Coordinator.DeleteExtRgn(Self);
      inherited Destroy
     end;

    procedure TExternalRegion.RgnChanged;
     begin
      if Rgn<>0 then
       DeleteObject(Rgn);
      Rgn:=0;
      Coordinator.ExtRgnMessage(Self,1)
     end;

    procedure TExternalRegion.SetName;
     begin
      FOldName:=Name;
      inherited SetName(Value);
      Coordinator.ExtRgnMessage(Self,2);
      FOldName:=''
     end;

    function TExternalRegion.GetRgn;
     begin
      if Rgn<>0 then
       begin
        Result:=CreateRectRgn(0,0,1,1);
        CombineRgn(Result,Rgn,Rgn,Rgn_Copy)
       end
      else
       if FOwnRegion then
        begin
         Rgn:=MakeRgn;
         Result:=CreateRectRgn(0,0,1,1);
         CombineRgn(Result,Rgn,Rgn,Rgn_Copy)
        end
       else
        Result:=MakeRgn
     end;

    function TExternalRegion.RgnSize;
     begin
      Result.CX:=-1;
      Result.CY:=-1
     end;

    {TTextExtRgn}

    constructor TTextExtRgn.Create;
     begin
      inherited Create(AOwner);
      FFont:=TFont.Create;
      FFont.OnChange:=FontChanged;
      FAngle:=0;
      FText:='';
      FWeight:=-1;
      FTransparent:=True
     end;

    destructor TTextExtRgn.Destroy;
     begin
      FFont.Free;
      inherited Destroy
     end;

    procedure TTextExtRgn.SetText;
     begin
      if Value<>FText then
       begin
        FText:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.SetAngle;
     begin
      if Value<>FAngle then
       begin
        FAngle:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.SetWeight;
     begin
      if Value<>FWeight then
       begin
        FWeight:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.SetTransparent;
     begin
      if Value<>FTransparent then
       begin
        FTransparent:=Value;
        RgnChanged
       end
     end;

    procedure TTextExtRgn.FontChanged;
     begin
      RgnChanged
     end;

    procedure TTextExtRgn.SetFont;
     begin
      FFont.Assign(Value);
      RgnChanged
     end;

    function TTextExtRgn.MakeRgn;
     var DC:HDC;
         LF:TLogFont;
         NewFont,OldFont:HFont;
      begin
       with LF,FFont do
        begin
         lfHeight:=Height;
         lfWidth:=0;
         lfEscapement:=FAngle;
         lfOrientation:=FAngle;
         if FWeight<0 then
          if fsBold in Style then
           lfWeight:=FW_Bold
          else
           lfWeight:=FW_Normal
         else
          lfWeight:=FWeight;
         lfItalic:=Byte(fsItalic in Style);
         lfUnderline:=Byte(fsUnderline in Style);
         lfStrikeOut:=Byte(fsStrikeOut in Style);
         lfCharSet:=Byte(CharSet);
         lfOutPrecision:=Out_Default_Precis;
         lfClipPrecision:=Clip_Default_Precis;
         lfQuality:=Default_Quality;
         case Pitch of
          fpVariable:lfPitchAndFamily:=Variable_Pitch;
          fpFixed:lfPitchAndFamily:=Fixed_Pitch
         else
          lfPitchAndFamily:=Default_Pitch
         end;
         StrPCopy(lfFaceName,Name)
        end;
       NewFont:=CreateFontIndirect(LF);
       DC:=CreateDC('DISPLAY',nil,nil,nil);
       OldFont:=SelectObject(DC,NewFont);
       BeginPath(DC);
       if FTransparent then
        SetBkMode(DC,Windows.Transparent);
       TextOut(DC,0,0,PChar(FText),Length(FText));
       EndPath(DC);
       Result:=PathToRegion(DC);
       SelectObject(DC,OldFont);
       DeleteObject(NewFont);
       DeleteDC(DC)
      end;

    {TPictExtRgn}

    constructor TPictExtRgn.Create;
     begin
      inherited Create(AOwner);
      FGlyph:=TBitmap.Create;
      FTransColor:=clOlive;
      FInverted:=False;
      FAlign[0]:=raResult;
      FAlign[1]:=raResult
     end;

    destructor TPictExtRgn.Destroy;
     begin
      FGlyph.Free;
      inherited Destroy
     end;

    procedure TPictExtRgn.SetGlyph;
     begin
      FGlyph.Assign(Value);
      RgnChanged
     end;

    procedure TPictExtRgn.SetTransColor;
     begin
      if Value<>FTransColor then
       begin
        FTransColor:=Value;
        RgnChanged
       end
     end;

    procedure TPictExtRgn.SetInverted;
     begin
      if FInverted<>Value then
       begin
        FInverted:=Value;
        RgnChanged
       end
     end;

    procedure TPictExtRgn.SetAlign;
     begin
      if Value<>FAlign[Index] then
       begin
        FAlign[Index]:=Value;
        if not FGlyph.Empty then
         RgnChanged
       end
     end;

    function TPictExtRgn.RgnSize;
     begin
      if not FGlyph.Empty and (FAlign[0]=raSource) then
       Result.CX:=FGlyph.Width
      else
       Result.CX:=-1;
      if not FGlyph.Empty and (FAlign[1]=raSource) then
       Result.CY:=FGlyph.Height
      else
       Result.CY:=-1
     end;

    function TPictExtRgn.MakeRgn;
     begin
      if not FGlyph.Empty then
       Result:=BitmapToRgn(FGlyph)
      else
       Result:=0
     end;

    function TPictExtRgn.BitmapToRgn;
     const AllocUnit=100;
      var BMP:TBitmap;
          MaxRects:Integer;
          HData:HGlobal;
          PData:PRgnData;
          CB,CR,CG,LR,LG,LB:Byte;
          P32:Pointer;
          X,X0,Y:Integer;
          P:PLongInt;
          PR:PRect;
          H:Hrgn;
       begin
        Result:=0;
        BMP:=TBitmap.Create;
        BMP.Assign(Bitmap);
        BMP.HandleType:=bmDIB;
        BMP.PixelFormat:=pf32bit;
        MaxRects:=AllocUnit;
        HData:=GlobalAlloc(GMem_Moveable,SizeOf(TRgnDataHeader)+SizeOf(TRect)*MaxRects);
        PData := GlobalLock(HData);
        PData^.RDH.dwSize:=SizeOf(TRgnDataHeader);
        PData^.RDH.iType:=RDH_Rectangles;
        PData^.RDH.nCount:=0;
        PData^.RDH.nRgnSize:=0;
        SetRect(PData^.RDH.rcBound,MaxInt,MaxInt,0,0);
        LR:=GetRValue(ColorToRGB(TransColor));
        LG:=GetGValue(ColorToRGB(TransColor));
        LB:=GetBValue(ColorToRGB(TransColor));
        for Y:=0 to Bitmap.Height-1 do
         begin
          X:=-1;
          P32:=BMP.ScanLine[Y];
          while X+1<Bitmap.Width do
           begin
            Inc(X);
            X0:=X;
            P:=PLongInt(P32);
            Inc(PChar(P),X*SizeOf(LongInt));
            while X<Bitmap.Width do
             begin
              CR:=GetBValue(P^);
              CG:=GetGValue(P^);
              CB:=GetRValue(P^);
              if ((CR=LR) and (CG=LG) and (CB=LB)) xor FInverted then
               Break;
              Inc(PChar(P),SizeOf(LongInt));
              Inc(X)
             end;
            if X>X0 then
             begin
              if PData^.RDH.nCount>=MaxRects then
               begin
                GlobalUnlock(HData);
                Inc(MaxRects,AllocUnit);
                HData:=GlobalReAlloc(HData,SizeOf(TRgnDataHeader)+SizeOf(TRect)*MaxRects,GMem_Moveable);
                PData:=GlobalLock(HData)
               end;
              PR:=@PData^.Buffer[PData^.RDH.nCount*SizeOf(TRect)];
              SetRect(PR^,X0,Y,X,Y+1);
              if X0<PData^.RDH.rcBound.Left then
               PData^.RDH.rcBound.Left:=X0;
              if Y<PData^.RDH.rcBound.Top then
               PData^.RDH.rcBound.Top:=Y;
              if X>PData^.RDH.rcBound.Right then
               PData^.RDH.rcBound.Right:=X;
              if Y+1>PData^.RDH.rcBound.Bottom then
               PData^.RDH.rcBound.Bottom:=Y+1;
              Inc(PData^.RDH.nCount);
              if PData^.RDH.nCount=2000 then
               begin
                H:=ExtCreateRegion(nil,SizeOf(TRgnDataHeader)+(SizeOf(TRect)*MaxRects),PData^);
                if Result<>0 then
                 begin
                  CombineRgn(Result,Result,H,RGN_OR);
                  DeleteObject(H);
                 end
                else
                 Result:=H;
                PData^.RDH.nCount:=0;
                SetRect(PData^.RDH.rcBound,MaxInt,MaxInt,0,0)
               end
             end
           end
         end;
        H:=ExtCreateRegion(nil,SizeOf(TRgnDataHeader)+(SizeOf(TRect)*MaxRects),PData^);
        if Result<>0 then
         begin
          CombineRgn(Result,Result,H,RGN_OR);
          DeleteObject(H);
         end
        else
         Result:=H;
        GlobalFree(HData);
        BMP.Free
       end;

    {TMacrosExtRgn}

    constructor TMacrosExtRgn.Create;
     begin
      inherited Create(AOwner);
      FRegion:=''
     end;

    procedure TMacrosExtRgn.SetRegion;
     begin
      if CompareStr(FRegion,Value)<>0 then
       begin
        FRegion:=Value;
        RgnChanged
       end
     end;

    function TMacrosExtRgn.MakeRgn;
     begin
      Result:=Coordinator.CreateRegion(FRegion,Name)
     end;

    procedure TMacrosExtRgn.ExtRgnMessage;
     var OldName,NewRgn:string;
         P:Integer;
      begin
       case Param of
        1:if Pos(UpperCase(ExtRgn.Name),UpperCase(FRegion))<>0 then
           Region:=Region;
        2:begin
           OldName:=UpperCase(ExtRgn.FOldName);
           if OldName<>'' then
            begin
             NewRgn:=FRegion;
             P:=Pos(OldName,UpperCase(NewRgn));
             while P<>0 do
              begin
               Delete(NewRgn,P,Length(OldName));
               Insert(ExtRgn.Name,NewRgn,P);
               P:=Pos(OldName,UpperCase(NewRgn))
              end;
             Region:=NewRgn
            end
          end
       end
      end;

    {TMenuHotSpot}

    constructor TMenuHotSpot.Create;
     begin
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of TMenuHotSpot component must be the TForm descendant');
      inherited Create(AOwner);
      FGlyph:=TBitmap.Create;
      Width:=16;
      Height:=16;
      FColors[0]:=clInactiveCaption;
      FColors[1]:=clActiveCaption;
      FTransparent:=True;
      ClickTime:=0;
      Next:=nil;
      Coordinator:=FindCoordinator(TForm(Owner));
      Coordinator.AddMenuHotSpot(Self)
     end;

    destructor TMenuHotSpot.Destroy;
     begin
      FGlyph.Free;
      Coordinator.DeleteMenuHotSpot(Self);
      inherited Destroy
     end;

    procedure TMenuHotSpot.SetGlyph;
     begin
      FGlyph.Assign(Value);
      if not FGlyph.Empty then
       begin
        Width:=FGlyph.Width;
        Height:=FGlyph.Height;
        FGlyph.Transparent:=FTransparent;
        if FTransparent then
         FGlyph.TransparentMode:=tmAuto
       end;
      Invalidate
     end;

    procedure TMenuHotSpot.SetTransparent;
     begin
      if Value<>FTransparent then
       begin
        FTransparent:=Value;
        if not FGlyph.Empty then
         begin
          FGlyph.TransparentMode:=tmAuto;
          FGlyph.Transparent:=Value
         end;
        Refresh
       end
     end;

    procedure TMenuHotSpot.SetColor;
     begin
      if Value<>FColors[Index] then
       begin
        FColors[Index]:=Value;
        if Transparent then
         Invalidate
       end
     end;

    procedure TMenuHotSpot.CallMenu;
     var Pt:TPoint;
      begin
       Pt:=(Owner as TControl).ClientToScreen(Point(Left,Top+Height));
       PopupMenu.Popup(Pt.X,Pt.Y)
      end;

    procedure TMenuHotSpot.MouseDown;
     begin
      if (Button=mbLeft) and Assigned(PopupMenu) then
       begin
        if (GetTickCount-ClickTime)>GetDoubleClickTime then
         CallMenu;
        ClickTime:=GetTickCount
       end
     end;

    procedure TMenuHotSpot.DblClick;
     var I:Integer;
      begin
       inherited DblClick;
       if Assigned(PopupMenu) then
        for I:=0 to PopupMenu.Items.Count-1 do
         if PopupMenu.Items[I].Default then
          begin
           PopupMenu.Items[I].Click;
           Break
          end
      end;

    procedure TMenuHotSpot.Paint;
     var Icon:HIcon;
      begin
       Canvas.Brush.Style:=bsSolid;
       if (Owner as TForm).Active then
        Canvas.Brush.Color:=FColors[1]
       else
        Canvas.Brush.Color:=FColors[0];
       Canvas.FillRect(ClientRect);
       if FGlyph.Empty then
        begin
         Width:=16;
         Height:=16;
         Icon:=LoadImage(HInstance,'MAINICON',Image_Icon,16,16,0);
         DrawIconEx(Canvas.Handle,0,0,Icon,16,16,0,0,DI_Normal);
         DeleteObject(Icon);
        end
       else
        Canvas.Draw(0,0,FGlyph)
      end;

    {TSystemButton}

    constructor TSystemButton.Create;
     begin
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of TSystemButton component must be the TForm descendant');
      inherited Create(AOwner);
      FKind:=sbClose;
      FAutoSize:=True;
      FOnDrawButton:=nil;
      FPressed:=False;
      FPressLoop:=False;
      FGlyph:=TBitmap.Create;
      inherited SetBounds(Left,Top,GetSystemMetrics(SM_CXSize)-2,GetSystemMetrics(SM_CYSize)-4);
      Next:=nil;
      Coordinator:=FindCoordinator(TForm(Owner));
      Coordinator.AddSystemButton(Self)
     end;

    destructor TSystemButton.Destroy;
     begin
      FGlyph.Free;
      Coordinator.DeleteSystemButton(Self);
      inherited Destroy
     end;

    procedure TSystemButton.SetKind;
     begin
      if Value<>FKind then
       begin
        FKind:=Value;
        Invalidate
       end
     end;

    procedure TSystemButton.SetAutoSize;
     begin
      if Value<>FAutoSize then
       begin
        FAutoSize:=Value;
        if Value then
         inherited SetBounds(Left,Top,GetSystemMetrics(SM_CXSize)-2,GetSystemMetrics(SM_CYSize)-4)
       end
     end;

    procedure TSystemButton.SetGlyph;
     begin
      FGlyph.Assign(Value);
      if not FGlyph.Empty then
       FKind:=sbGlyph;
      if FKind=sbGlyph then
       Invalidate
     end;

    procedure TSystemButton.Paint;
     var BState:Integer;
         R:TRect;
      begin
       if FKind<>sbOwnerDraw then
        begin
         R:=Rect(0,0,Width,Height);
         if not Enabled then
          BState:=DFCS_Inactive
         else
          if Pressed then
           BState:=DFCS_Pushed
          else
           BState:=0;
         case FKind of
          sbClose:DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionClose or BState);
          sbMinimize:if (Owner as TForm).WindowState=wsMinimized then
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionRestore or BState)
                     else
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionMin or BState);
          sbMaximize:if (Owner as TForm).WindowState=wsMaximized then
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionRestore or BState)
                     else
                      DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionMax or BState);
          sbHelp:DrawFrameControl(Canvas.Handle,R,DFC_Caption,DFCS_CaptionHelp or BState);
          sbGlyph:begin
                   DrawFrameControl(Canvas.Handle,R,DFC_Button,DFCS_ButtonPush or BState);
                   if not FGlyph.Empty then
                    Canvas.Draw((Width-FGlyph.Width) div 2,(Height-FGlyph.Height) div 2,FGlyph)
                  end;
          sbOwnerDrawFace:begin
                           DrawFrameControl(Canvas.Handle,R,DFC_Button,DFCS_ButtonPush or BState);
                           if Assigned(FOnDrawButton) then
                            FOnDrawButton(Self)
                          end
         end
        end
       else
        if Assigned(FOnDrawButton) then
         FOnDrawButton(Self)
      end;

    procedure TSystemButton.SetBounds;
     begin
      if (AWidth<>Width) or (AHeight<>Height) then
       FAutoSize:=False;
      inherited SetBounds(ALeft,ATop,AWidth,AHeight)
     end;

    procedure TSystemButton.CMEnabledChanged;
     begin
      inherited
      Refresh
     end;

    procedure TSystemButton.MouseDown;
     begin
      inherited MouseDown(Button,Shift,X,Y);
      if Button=mbLeft then
       begin
        FPressed:=True;
        Refresh;
        FPressLoop:=True
       end
     end;

    procedure TSystemButton.MouseMove;
     begin
      inherited MouseMove(Shift,X,Y);
      if FPressLoop then
       if (X>=0) and (X<ClientWidth) and (Y>=0) and (Y<ClientWidth) then
        begin
         if not FPressed then
          begin
           FPressed:=True;
           Refresh
          end
        end
       else
        if FPressed then
         begin
          FPressed:=False;
          Refresh
         end
     end;

    procedure TSystemButton.MouseUp;
     begin
      inherited MouseUp(Button,Shift,X,Y);
      if FPressLoop and (Button=mbLeft) then
       begin
        FPressLoop:=False;
        FPressed:=False;
        Refresh;
        if  (X>=0) and (X<ClientWidth) and (Y>=0) and (Y<ClientWidth) then
         case FKind of
          sbClose:SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Close,-1);
          sbMinimize:if (Owner as TForm).WindowState=wsMinimized then
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Restore,-1)
                     else
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Minimize,-1);
          sbMaximize:if (Owner as TForm).WindowState=wsMaximized then
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Restore,-1)
                     else
                      SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_Maximize,-1);
          sbHelp:SendMessage((Owner as TForm).Handle,WM_SysCommand,SC_ContextHelp,-1)
         end
       end
     end;

    procedure TSystemButton.Loaded;
     begin
      inherited Loaded;
      if AutoSize then
       inherited SetBounds(Left,Top,GetSystemMetrics(SM_CXSize)-2,GetSystemMetrics(SM_CYSize)-4)
     end;

    {TCaptionText}

    constructor TCaptionText.Create;
     begin
      if not (AOwner is TForm) then
       raise EOwnerMustBeForm.Create('The owner of TCaptionText component must be TForm descendant');
      inherited Create(AOwner);
      FColors[0]:=clInactiveCaptionText;
      FColors[1]:=clCaptionText;
      FAutoSize:=True;
      FAngle:=0;
      Next:=nil;
      Coordinator:=FindCoordinator(TForm(Owner));
      Coordinator.AddCaptionText(Self)
     end;

    destructor TCaptionText.Destroy;
     begin
      Coordinator.DeleteCaptionText(Self);
      inherited Destroy
     end;

    procedure TCaptionText.SetAngle;
     begin
      if Value<>FAngle then
       begin
        while Value<0 do
         Inc(Value,3600);
        while Value>3599 do
         Dec(Value,3600);
        FAngle:=Value;
        if FAutoSize then
         SetSize;
        Invalidate
       end
     end;

    procedure TCaptionText.SetAutoSize;
     begin
      if Value<>FAutoSize then
       begin
        FAutoSize:=Value;
        if Value then
         SetSize
       end
      end;

    procedure TCaptionText.SetColor;
     begin
      if FColors[Index]<>Value then
       begin
        FColors[Index]:=Value;
        Invalidate
       end
     end;

    procedure TCaptionText.CMTextChanged;
     begin
      inherited;
      if FAutoSize then
       SetSize;
      Invalidate
     end;

    procedure TCaptionText.CMFontChanged;
     begin
      inherited;
      if AutoSize then
       SetSize
     end;

    function TCaptionText.CalcSize;
     var DC:HDC;
         Sz:TSize;
         OldFont:HFont;
         RAngle:Double;
      begin
       DC:=CreateDC('DISPLAY',nil,nil,nil);
       OldFont:=SelectObject(DC,Font.Handle);
       GetTextExtentPoint32(DC,PChar(Caption),Length(Caption),Sz);
       if FAngle=0 then
        Result:=Sz
       else
        begin
         RAngle:=FAngle/1800*Pi;
         Result.CX:=Round(Abs(Sz.CX*Cos(RAngle))+Abs(Sz.CY*Sin(RAngle)));
         Result.CY:=Round(Abs(Sz.CX*Sin(RAngle))+Abs(Sz.CY*Cos(RAngle)))
        end;
       SelectObject(DC,OldFont);
       DeleteDC(DC)
      end;

    procedure TCaptionText.SetSize;
     var Sz:TSize;
      begin
       Sz:=CalcSize;
       Width:=Sz.CX;
       Height:=Sz.CY
      end;

    procedure TCaptionText.Paint;
     var LF:TLogFont;
         Sz:TSize;
         X,Y:Integer;
         NewFont,OldFont:HFont;
         SA,CA:Extended;
      begin
       with LF,Font do
        begin
         lfHeight:=Height;
         lfWidth:=0;
         lfEscapement:=FAngle;
         lfOrientation:=FAngle;
         if fsBold in Style then
          lfWeight:=FW_Bold
         else
          lfWeight:=FW_Normal;
         lfItalic:=Byte(fsItalic in Style);
         lfUnderline:=Byte(fsUnderline in Style);
         lfStrikeOut:=Byte(fsStrikeOut in Style);
         lfCharSet:=Byte(CharSet);
         lfOutPrecision:=Out_Default_Precis;
         lfClipPrecision:=Clip_Default_Precis;
         lfQuality:=Default_Quality;
         case Pitch of
          fpVariable:lfPitchAndFamily:=Variable_Pitch;
          fpFixed:lfPitchAndFamily:=Fixed_Pitch
         else
          lfPitchAndFamily:=Default_Pitch
         end;
         StrPCopy(lfFaceName,Name)
        end;
       SetBkMode(Canvas.Handle,Transparent);
       NewFont:=CreateFontIndirect(LF);
       OldFont:=SelectObject(Canvas.Handle,NewFont);
       GetTextExtentPoint32(Canvas.Handle,PChar(Caption),Length(Caption),Sz);
       SinCos(FAngle/1800*Pi,SA,CA);
       case FAngle of
        0:begin
           X:=0;
           Y:=0
          end;
        1..899:begin
                X:=0;
                Y:=Round(Sz.CX*SA)
               end;
        900:begin
             X:=0;
             Y:=Sz.CX
            end;
        901..1799:begin
                   X:=Round(Abs(Sz.CX*CA));
                   Y:=Round(Sz.CX*SA+Abs(Sz.CY*CA))
                  end;
        1800:begin
              X:=Sz.CX;
              Y:=Sz.CY
             end;
        1801..2699:begin
                    X:=Round(Abs(Sz.CX*CA)+Abs(Sz.CY*SA));
                    Y:=Round(Abs(Sz.CY*CA))
                   end;
        2700:begin
              X:=Sz.CY;
              Y:=0
             end
       else
        begin
         X:=Round(Abs(Sz.CY*SA));
         Y:=0
        end
       end;
       SetTextColor(Canvas.Handle,ColorToRGB(FColors[Integer((Owner as TForm).Active)]));
       Canvas.TextOut(X,Y,Caption);
       SelectObject(Canvas.Handle,OldFont);
       DeleteObject(NewFont)
      end;

    {TFRManager}

    constructor TFRManager.Create;
     var I:Integer;
      begin
       if not (AOwner is TForm) then
        raise EOwnerMustBeForm.Create('The owner of TFRManager component must be the TForm descendant');
       with AOwner do
        for I:=0 to ComponentCount-1 do
         if (Components[I] is TFRManager) and (Components[I]<>Self) then
          raise EDuplicateComponent.Create('Only one TFRManager component on a form is allowed');
       inherited Create(AOwner);
       Coordinator:=FindCoordinator(TForm(Owner));
       Coordinator.AssignManager;
       CallCount:=0
      end;

    destructor TFRManager.Destroy;
     begin
      Coordinator.RemoveManager;
      inherited Destroy
     end;

    procedure TFRManager.Activate;
     begin
      if CallCount>0 then
       begin
        Dec(CallCount);
        if CallCount=0 then
         Coordinator.Activate
       end
     end;

    procedure TFRManager.Deactivate;
     begin
      Inc(CallCount);
      if CallCount=1 then
       Coordinator.Deactivate
     end;

    procedure TFRManager.StayOnTop;
     begin
      Deactivate;
      if Stay then
       TForm(Owner).FormStyle:=fsStayOnTop
      else
       TForm(Owner).FormStyle:=fsNormal;
      Activate
     end;

    {TFRCoordinator}

    constructor TFRCoordinator.Create;
     var P:TFRCoordinator;
      begin
       {$IfDef Unregistred}
       if not (csDesigning in AForm.ComponentState) then
        with TFormStop.Create(nil) do
         begin
          ShowModal;
          Free
         end;
       {$EndIf}
       inherited Create;
       Form:=AForm;
       RefCount:=0;
       FormRegion:=nil;
       CaptionRegion:=nil;
       Glass:=nil;
       ExtRgn:=nil;
       HotSpot:=nil;
       SysBtn:=nil;
       CptText:=nil;
       Next:=nil;
       ScrImage:=nil;
       FDelayTime:=300;
       FDrawOnDesigning:=False;
       Moving:=False;
       NameList:=TStringList.Create;
       if Assigned(CHead) then
        begin
         P:=CHead;
         while Assigned(P.Next) do
          P:=P.Next;
         P.Next:=Self
        end
       else
        CHead:=Self;
       HookForm
      end;

    destructor TFRCoordinator.Destroy;
     var P:TFRCoordinator;
      begin
       UnhookForm;
       NameList.Free;
       if CHead=Self then
        CHead:=Next
       else
        begin
         P:=CHead;
         while P.Next<>Self do
          P:=P.Next;
         P.Next:=Next
        end;
       inherited Destroy
      end;

    procedure TFRCoordinator.DecRefCount;
     begin
      Dec(RefCount);
      if RefCount<=0 then
       Free
     end;

    procedure TFRCoordinator.AssignFormRegion;
     begin
      FormRegion:=FRgn;
      Inc(RefCount)
     end;

    procedure TFRCoordinator.RemoveFormRegion;
     begin
      if Assigned(FormRegion) then
       begin
        FormRegion:=nil;
        DecRefCount
       end
     end;

    procedure TFRCoordinator.AssignCaptionRegion;
     begin
      CaptionRegion:=CRgn;
      Inc(RefCount)
     end;

    procedure TFRCoordinator.RemoveCaptionRegion;
     begin
      if Assigned(CaptionRegion) then
       begin
        CaptionRegion:=nil;
        DecRefCount
       end
     end;

    procedure TFRCoordinator.AddGlassRegion;
     var P:TGlassRegion;
      begin
       if Assigned(Glass) then
        begin
         P:=Glass;
         while Assigned(P.Next) do
          P:=P.Next;
         P.Next:=GlassRegion
        end
       else
        Glass:=GlassRegion;
       Inc(RefCount);
       if not Assigned(ScrImage) then
        begin
         ScrImage:=TBitmap.Create;
         ScrImage.Width:=GetSystemMetrics(SM_CXScreen);
         ScrImage.Height:=GetSystemMetrics(SM_CYScreen);
         ScrImage.PixelFormat:=pf24Bit
        end
      end;

    procedure TFRCoordinator.DeleteGlassRegion;
     var P:TGlassRegion;
      begin
       if GlassRegion=Glass then
        Glass:=Glass.Next
       else
        begin
         P:=Glass;
         while P.Next<>GlassRegion do
          P:=P.Next;
         P.Next:=P.Next.Next
        end;
       if not Assigned(Glass) then
        ScrImage.Free;
       DecRefCount
      end;

    procedure TFRCoordinator.AddExtRgn;
     var P:TExternalRegion;
      begin
       if Assigned(ExtRgn) then
        begin
         P:=ExtRgn;
         while Assigned(P.Next) do
          P:=P.Next;
         P.Next:=ExternalRegion
        end
       else
        ExtRgn:=ExternalRegion;
       Inc(RefCount)
      end;

    procedure TFRCoordinator.DeleteExtRgn;
     var P:TExternalRegion;
      begin
       if ExternalRegion=ExtRgn then
        ExtRgn:=ExtRgn.Next
       else
        begin
         P:=ExtRgn;
         while P.Next<>ExternalRegion do
          P:=P.Next;
         P.Next:=P.Next.Next
        end;
       DecRefCount
      end;

    procedure TFRCoordinator.AddMenuHotSpot;
     var P:TMenuHotSpot;
      begin
       if Assigned(HotSpot) then
        begin
         P:=HotSpot;
         while Assigned(P.Next) do
          P:=P.Next;
         P.Next:=MenuHotSpot
        end
       else
        HotSpot:=MenuHotSpot;
       Inc(RefCount)
      end;

    procedure TFRCoordinator.DeleteMenuHotSpot;
     var P:TMenuHotSpot;
      begin
       if MenuHotSpot=HotSpot then
        HotSpot:=HotSpot.Next
       else
        begin
         P:=HotSpot;
         while P.Next<>MenuHotSpot do
          P:=P.Next;
         P.Next:=P.Next.Next
        end;
       DecRefCount
      end;

    procedure TFRCoordinator.AddSystemButton;
     var P:TSystemButton;
      begin
       if Assigned(SysBtn) then
        begin
         P:=SysBtn;
         while Assigned(P.Next) do
          P:=P.Next;
         P.Next:=SystemButton
        end
       else
        SysBtn:=SystemButton;
       Inc(RefCount)
      end;

    procedure TFRCoordinator.DeleteSystemButton;
     var P:TSystemButton;
      begin
       if SystemButton=SysBtn then
        SysBtn:=SysBtn.Next
       else
        begin
         P:=SysBtn;
         while P.Next<>SystemButton do
          P:=P.Next;
         P.Next:=P.Next.Next
        end;
       DecRefCount
      end;

    procedure TFRCoordinator.AddCaptionText;
     var P:TCaptionText;
      begin
       if Assigned(CptText) then
        begin
         P:=CptText;
         while Assigned(P.Next) do
          P:=P.Next;
         P.Next:=CaptionText
        end
       else
        CptText:=CaptionText;
       Inc(RefCount)
      end;

    procedure TFRCoordinator.DeleteCaptionText;
     var P:TCaptionText;
      begin
       if CaptionText=CptText then
        CptText:=CptText.Next
       else
        begin
         P:=CptText;
         while P.Next<>CaptionText do
          P:=P.Next;
         P.Next:=P.Next.Next
        end;
       DecRefCount
      end;

    procedure TFRCoordinator.AssignManager;
     begin
      Inc(RefCount)
     end;

    procedure TFRCoordinator.RemoveManager;
     begin
      DecRefCount
     end;

    procedure TFRCoordinator.Activate;
     var PGlass:TGlassRegion;
      begin
       HookForm;
       if Assigned(FormRegion) then
        FormRegion.Refresh;
       if Assigned(CaptionRegion) then
        CaptionRegion.Refresh;
       PGlass:=Glass;
       while Assigned(PGlass) do
        begin
         PGlass.Refresh;
         PGlass:=PGlass.Next
        end
      end;

    procedure TFRCoordinator.Deactivate;
     begin
      UnhookForm
     end;

    procedure TFRCoordinator.HookWndProc;
     var OwnDC:Boolean;
         PS:TPaintStruct;
         PMenu:TMenuHotSpot;
         SBtn:TSystemButton;
         CText:TCaptionText;
         PGlass:TGlassRegion;
         ERgn:TExternalRegion;
         CW,CH,CX,CY:Integer;
         TicksNow:Cardinal;
         BM2,BM3:TBitmap;
         DC:HDC;
         WasActive:Boolean;
         PrevWnd:HWnd;
      begin
       case Message.Msg of
        WM_Size:begin
                 ERgn:=ExtRgn;
                 while Assigned(ERgn) do
                  begin
                   if ERgn is TMacrosExtRgn then
                    ERgn.RgnChanged;
                   ERgn:=ERgn.Next
                  end;
                 if Assigned(FormRegion) then
                  FormRegion.SizeChanged;
                 if Assigned(CaptionRegion) then
                  CaptionRegion.SizeChanged;
                 PGlass:=Glass;
                 while Assigned(PGlass) do
                  begin
                   PGlass.SizeChanged;
                   PGlass:=PGlass.Next
                  end;
                 CallDefault(Message);
                 SBtn:=SysBtn;
                 while Assigned(SBtn) do
                  begin
                   if SBtn.FKind in [sbMinimize,sbMaximize,sbOwnerDraw,sbOwnerDrawFace] then
                    SBtn.Refresh;
                   SBtn:=SBtn.Next
                  end
                end;
        WM_Activate:begin
                     CallDefault(Message);
                     if Assigned(CaptionRegion) then
                      CaptionRegion.InvalidateCaption;
                     if Assigned(FormRegion) then
                      with FormRegion do
                       if (csDesigning in ComponentState) and (Rgn<>0) and DrawOnDesigning then
                        PostMessage(Form.Handle,WM_NCPaint,0,0);
                     PMenu:=HotSpot;
                     while Assigned(PMenu) do
                      begin
                       PMenu.Refresh;
                       PMenu:=PMenu.Next
                      end;
                     CText:=CptText;
                     while Assigned(CText) do
                      begin
                       CText.Refresh;
                       CText:=CText.Next
                        end
                    end;
        WM_Paint:begin
                  BM3:=nil;
                  OwnDC:=Message.WParam=0;
                  if Assigned(Glass) and (FDrawOnDesigning or not (csDesigning in Form.ComponentState)) then
                   with Form do
                    begin
                     if not OwnDC then
                      raise EPaintAlreadyHooked.Create('WM_Paint is hooked before FormRgn. See help to resolve this conflict');
                     CW:=ClientWidth;
                     CH:=ClientHeight;
                     CX:=ClientOrigin.X;
                     CY:=ClientOrigin.Y;
                     WasActive:=GetActiveWindow=Form.Handle;
                     if not Moving then
                      begin
                       PrevWnd:=GetNextWindow(Handle,GW_HWndPrev);
                       SetWindowPos(Handle,0,0,0,0,0,SWP_NoMove or SWP_NoSize or SWP_NoZOrder or SWP_NoActivate or SWP_HideWindow);
                       if WasActive then
                        SetActiveWindow(0);
                       TicksNow:=GetTickCount;
                       repeat
                        Application.ProcessMessages
                       until GetTickCount-TicksNow>=FDelayTime;
                       DC:=GetDC(0);
                       BitBlt(ScrImage.Canvas.Handle,0,0,ScrImage.Width,ScrImage.Height,DC,0,0,SrcCopy);
                       ReleaseDC(0,DC)
                      end;
                     BM2:=TBitmap.Create;
                     BM2.Width:=CW+1;
                     BM2.Height:=CH+1;
                     BM2.PixelFormat:=pf24bit;
                     BM2.Canvas.Draw(-CX,-CY,ScrImage);
                     BM3:=TBitmap.Create;
                     BM3.Width:=CW+1;
                     BM3.Height:=CH+1;
                     BM3.PixelFormat:=pf24bit;
                     BM3.Canvas.Brush.Color:=Form.Color;
                     BM3.Canvas.FillRect(Rect(0,0,CW+1,CH+1));
                     PGlass:=Glass;
                     while Assigned(PGlass) do
                      begin
                       PGlass.DrawStainedGlass(BM2,BM3);
                       PGlass:=PGlass.Next
                      end;
                     BM2.Free;
                     if not Moving then
                      if WasActive then
                       SetWindowPos(Handle,0,0,0,0,0,SWP_NoMove or SWP_NoSize or SWP_ShowWindow)
                      else
                       SetWindowPos(Handle,PrevWnd,0,0,0,0,SWP_NoMove or SWP_NoSize or SWP_ShowWindow or SWP_NoActivate)
                    end;
                  if OwnDC then
                   Message.WParam:=BeginPaint(Form.Handle,PS);
                  if (csDesigning in Form.ComponentState) and not FDrawOnDesigning and Assigned(Glass) then
                   begin
                    PGlass:=Glass;
                    BM2:=TBitmap.Create;
                    BM2.Width:=Form.ClientWidth+1;
                    BM2.Height:=Form.ClientHeight+1;
                    BM2.Canvas.Brush.Color:=Form.Color;
                    BM2.Canvas.FillRect(Rect(0,0,BM2.Width,BM2.Height));
                    while Assigned(PGlass) do
                     begin
                      PGlass.DraftGlass(BM2);
                      PGlass:=PGlass.Next
                     end;
                    BitBlt(Message.WParam,0,0,BM2.Width,BM2.Height,BM2.Canvas.Handle,0,0,SrcCopy);
                    BM2.Destroy;
                   end;
                  if Assigned(BM3) then
                   begin
                    BitBlt(Message.WParam,0,0,BM3.Width,BM3.Height,BM3.Canvas.Handle,0,0,SrcCopy);
                    BM3.Destroy
                   end;
                  if Assigned(CaptionRegion) then
                   CaptionRegion.DrawCaption(Message.WParam);
                  CallDefault(Message);
                  if OwnDC then
                   EndPaint(Form.Handle,PS);
                  if Assigned(FormRegion) then
                   with FormRegion do
                    if (csDesigning in ComponentState) and (Rgn<>0) and DrawOnDesigning then
                     begin
                      PostMessage(Form.Handle,WM_NCActivate,Integer(Form.Active),0);
                      PostMessage(Form.Handle,WM_NCPaint,0,0)
                     end
                 end;
        WM_EraseBkgnd:if Assigned(Glass) and (FDrawOnDesigning or not (csDesigning in Form.ComponentState)) then
                       Message.Result:=1
                      else
                       CallDefault(Message);
        WM_WindowPosChanged:begin
                             CallDefault(Message);
                             if Assigned(Glass) and (FDrawOnDesigning or not (csDesigning in Form.ComponentState)) then
                              Form.Invalidate
                            end;
        WM_DisplayChange:begin
                          CallDefault(Message);
                          if Assigned(ScrImage) then
                           begin
                            ScrImage.Width:=Message.LParamLo;
                            ScrImage.Height:=Message.LParamHi;
                            Form.Invalidate
                           end
                         end;
        WM_NCPaint:begin
                    CallDefault(Message);
                    if Assigned(FormRegion) then
                     FormRegion.HatchRegion
                   end;

        WM_NCHitTest:begin
                      CallDefault(Message);
                      if not (csDesigning in Form.ComponentState) then
                       if Message.Result=HTClient then
                        if Assigned(FormRegion) and FormRegion.ClientDrag then
                         Message.Result:=HTCaption
                        else
                         if Assigned(CaptionRegion) and (CaptionRegion.PointArea(Message.LParam)<>0) then
                          Message.Result:=HTCaption
                     end;
        WM_NCLButtonDown:if csDesigning in Form.ComponentState then
                          CallDefault(Message)
                         else
                          if Assigned(CaptionRegion) and (CaptionRegion.PointArea(Message.LParam)=2) then
                           CaptionRegion.Popup(-1,-1)
                          else
                           CallDefault(Message);
        WM_NCLButtonDblClk:if csDesigning in Form.ComponentState then
                            CallDefault(Message)
                           else
                            if Assigned(CaptionRegion) and (Message.WParam=HTCaption) then
                             if CaptionRegion.PointArea(Message.LParam)=1 then
                              if (Form.BorderStyle=bsNone) and (biMaximize in Form.BorderIcons) then
                               begin
                                if Form.WindowState=wsMaximized then
                                 Form.WindowState:=wsNormal
                                else
                                 Form.WindowState:=wsMaximized;
                                Message.Result:=0
                               end
                              else
                               CallDefault(Message)
                             else
                              begin
                               CaptionRegion.DefaultItem;
                               Message.Result:=0
                              end
                            else
                             CallDefault(Message);
        WM_NCRButtonDown:if csDesigning in Form.ComponentState then
                          CallDefault(Message)
                         else
                          if Assigned(CaptionRegion) and (Message.WParam=HTCaption) then
                           begin
                            CaptionRegion.Popup(Message.LParamLo,Message.LParamHi);
                            Message.Result:=0
                           end
                          else
                           CallDefault(Message);
        WM_KeyDown,WM_SysKeyDown:if csDesigning in Form.ComponentState then
                                  CallDefault(Message)
                                 else
                                  if Assigned(CaptionRegion) and Assigned(CaptionRegion.FPopupMenu) and CaptionRegion.FPopupMenu.IsShortCut(TWMKey(Message)) then
                                   Message.Result:=0
                                  else
                                   begin
                                    PMenu:=HotSpot;
                                    while Assigned(PMenu) do
                                     begin
                                      if Assigned(PMenu.PopupMenu) and PMenu.PopupMenu.IsShortCut(TWMKey(Message)) then
                                       begin
                                        Message.Result:=0;
                                        Exit
                                       end;
                                      PMenu:=PMenu.Next
                                     end;
                                    CallDefault(Message)
                                   end;
        WM_SysChar:if csDesigning in Form.ComponentState then
                    CallDefault(Message)
                   else
                    if Message.WParam=VK_Space then
                     if Assigned(CaptionRegion) and CaptionRegion.FSysMenuPicture.Visible and Assigned(CaptionRegion.FPopupMenu) then
                      begin
                       CaptionRegion.Popup(-1,-1);
                       Message.Result:=0
                      end
                     else
                      begin
                       PMenu:=HotSpot;
                       while Assigned(PMenu) do
                        begin
                         if Assigned(PMenu.PopupMenu) then
                          begin
                           PMenu.CallMenu;
                           Message.Result:=0;
                           Exit
                          end;
                         PMenu:=PMenu.Next
                        end;
                       CallDefault(Message)
                      end
                    else
                     CallDefault(Message);
        WM_EnterSizeMove:begin
                          Moving:=True;
                          CallDefault(Message)
                         end;
        WM_ExitSizeMove:begin
                         CallDefault(Message);
                         Moving:=False
                        end
       else
        CallDefault(Message)
       end
      end;

    procedure TFRCoordinator.HookForm;
     begin
      if not Assigned(Form) then
       Exit;
      OldWndProc:=TFarProc(GetWindowLong(Form.Handle,GWL_WndProc));
      NewWndProc:=MakeObjectInstance(HookWndProc);
      SetWindowLong(Form.Handle,GWL_WndProc,LongInt(NewWndProc))
     end;

    procedure TFRCoordinator.UnhookForm;
     begin
      if Assigned(Form) and Assigned(OldWndProc) and not (csDestroying in Form.ComponentState) and Form.HandleAllocated then
       SetWindowLong(Form.Handle,GWL_WndProc,LongInt(OldWndProc));
      if Assigned(NewWndProc) then
       FreeObjectInstance(NewWndProc);
      NewWndProc:=nil;
      OldWndProc:=nil
     end;

    procedure TFRCoordinator.ExtRgnMessage;
     var ERgn:TExternalRegion;
         PGlass:TGlassRegion;
      begin
       ERgn:=ExtRgn;
       while Assigned(ERgn) do
        begin
         if ERgn is TMacrosExtRgn then
          TMacrosExtRgn(ERgn).ExtRgnMessage(ExtRgn,Param);
         ERgn:=ERgn.Next
        end;
       PGlass:=Glass;
       while Assigned(PGlass) do
        begin
         PGlass.ExtRgnMessage(ExtRgn,Param);
         PGlass:=PGlass.Next
        end;
       if Assigned(FormRegion) then
        FormRegion.ExtRgnMessage(ExtRgn,Param);
       if Assigned(CaptionRegion) then
        CaptionRegion.ExtRgnMessage(ExtRgn,Param)
      end;

    function TFRCoordinator.CreateRegion;
     var S:string;
         Comb:array[1..15] of Integer;
         P,N,C:Integer;
         R2:array[1..16] of HRgn;
         First:array[1..15] of Boolean;

      procedure ProcessString(S:string);
       var I,P,CN:Integer;
           RR:array[1..6] of Integer;
           Ch,ChV,ChH:Char;
           Pts:PPointsArray;
           PE:TExternalRegion;
           R:TRect;
           X,Y:Integer;
           Found:Boolean;
           Sz:TSize;

        function Trans(var P:Integer;const S:string):Integer;
         forward;

        function Factor(var P:Integer;const S:string):Integer;
         forward;

        function Expr(var P:Integer;const S:string):Integer;
         begin
          Result:=Trans(P,S);
          while S[P] in ['+','-'] do
           case S[P] of
            '+':begin
                 Inc(P);
                 Inc(Result,Trans(P,S))
                end;
            '-':begin
                 Inc(P);
                 Dec(Result,Trans(P,S))
                end
           end
         end;

        function Trans;
         begin
          Result:=Factor(P,S);
          while S[P] in ['*','/'] do
           case S[P] of
            '*':begin
                 Inc(P);
                 Result:=Result*Factor(P,S);
                end;
            '/':begin
                 Inc(P);
                 Result:=Result div Factor(P,S)
                end
           end
         end;

        function Factor;
         var J,D:Integer;
          begin
           case UpCase(S[P]) of
            '(':begin
                 Inc(P);
                 Result:=Expr(P,S);
                 if S[P]=')' then
                  Inc(P)
                 else
                  raise ECannotTranslate(SelfName+', line'+IntToStr(N)+': '')'' expected in position '+IntToStr(P))
                end;
            '-':begin
                 Inc(P);
                 Result:=-Factor(P,S)
                end;
            '0'..'9':begin
                      Val(Copy(S,P,255),Result,D);
                      J:=D;
                      Val(Copy(S,P,J-1),Result,D);
                      P:=P+J-1
                     end;
            'B':begin
                 Inc(P);
                 with Form do
                  Result:=ClientHeight+ClientOrigin.Y-Top-GetYOrigin
                end;
            'C':begin
                 Inc(P);
                 with Form do
                  Result:=ClientOrigin.Y-Top-GetYOrigin
                end;
            'H':begin
                 Inc(P);
                 Result:=Form.Height-DesignHeight
                end;
            'R':begin
                 Inc(P);
                 with Form do
                  Result:=ClientWidth+ClientOrigin.X-Left-GetXOrigin
                end;
            'W':begin
                 Inc(P);
                 Result:=Form.Width-DesignWidth
                end;
            'X':begin
                 Inc(P);
                 with Form do
                  Result:=ClientOrigin.X-Left-GetXOrigin
                end;
            'Y':begin
                 Inc(P);
                 with Form do
                  if BorderStyle in [bsToolWindow,bsSizeToolWin] then
                   Result:=ClientOrigin.Y-Top-SmCapt-GetYOrigin
                  else
                   Result:=ClientOrigin.Y-Top-NCapt-GetYOrigin
                end
           else
            raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Syntax error in position '+IntToStr(P))
           end
          end;

        function GetValue(const S:string):Integer;
         var P:Integer;
          begin
           P:=1;
           Result:=Expr(P,S+' ')
          end;

        function MakeRect(S:string):TRect;
         var I,CN:Integer;
          begin
           CN:=0;
           for I:=1 to Length(S) do
            if S[I]=',' then
             Inc(CN);
           if CN<>3 then
            raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Invalid number of parameters');
           I:=Pos(',',S);
           Result.Left:=GetValue(Copy(S,1,I-1));
           Delete(S,1,I);
           I:=Pos(',',S);
           Result.Top:=GetValue(Copy(S,1,I-1));
           Delete(S,1,I);
           I:=Pos(',',S);
           Result.Right:=GetValue(Copy(S,1,I-1));
           Delete(S,1,I);
           Result.Bottom:=GetValue(S)
          end;

         begin
          R2[C+1]:=0;
          Ch:=S[1];
          Delete(S,1,1);
          case Ch of
           'A':Comb[C]:=Rgn_And;
           'D':Comb[C]:=Rgn_Diff;
           'E':R2[C+1]:=CreateEllipticRgnIndirect(MakeRect(S));
           'L','W':begin
                    CN:=0;
                    for I:=1 to Length(S) do
                     if S[I]=',' then
                      Inc(CN);
                    if (CN<5) or not Odd(CN) then
                     raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Invalid number of parameters');
                    CN:=(CN+1) div 2;
                    GetMem(Pts,SizeOf(TPoint)*CN);
                    try
                     for I:=1 to CN-1 do
                      begin
                       P:=Pos(',',S);
                       Pts^[I].X:=GetValue(Copy(S,1,P-1));
                       Delete(S,1,P);
                       P:=Pos(',',S);
                       Pts^[I].Y:=GetValue(Copy(S,1,P-1));
                       Delete(S,1,P)
                      end;
                     P:=Pos(',',S);
                     Pts^[CN].X:=GetValue(Copy(S,1,P-1));
                     Delete(S,1,P);
                     Pts^[CN].Y:=GetValue(S);
                     if Ch='L' then
                      R2[C+1]:=CreatePolygonRgn(Pts^,CN,Alternate)
                     else
                      R2[C+1]:=CreatePolygonRgn(Pts^,CN,Winding)
                    finally
                     FreeMem(Pts)
                    end
                   end;
           'O':Comb[C]:=Rgn_Or;
           'R':R2[C+1]:=CreateRectRgnIndirect(MakeRect(S));
           'T':if Length(S)<3 then
                raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Syntax error')
               else
                begin
                 ChH:=S[1];
                 ChV:=S[2];
                 Delete(S,1,2);
                 P:=Pos(',',S);
                 if P=0 then
                  raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Invalid number of parameters');
                 X:=GetValue(Copy(S,1,P-1));
                 Delete(S,1,P);
                 P:=Pos(',',S);
                 if P=0 then
                  raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Invalid number of parameters');
                 Y:=GetValue(Copy(S,1,P-1));
                 Delete(S,1,P);
                 R2[C+1]:=0;
                 PE:=ExtRgn;
                 Found:=False;
                 for I:=0 to NameList.Count-1 do
                  if CompareText(S,NameList[I])=0 then
                   raise EExtRgnCrossLink.Create(SelfName+' recursively links to itself in line '+IntToStr(N));
                 while Assigned(PE) do
                  if CompareText(PE.Name,S)=0 then
                   begin
                    R2[C+1]:=PE.GetRgn;
                    Sz:=PE.RgnSize;
                    Found:=True;
                    Break
                   end
                  else
                   PE:=PE.Next;
                 if not Found then
                  ShowMessage(SelfName+', line '+IntToStr(N)+': External region '+S+' not found')
                 else
                  if R2[C+1]<>0 then
                   begin
                    GetRgnBox(R2[C+1],R);
                    if Sz.CX<>-1 then
                     begin
                      R.Left:=0;
                      R.Right:=Sz.CX
                     end;
                    if Sz.CY<>-1 then
                     begin
                      R.Top:=0;
                      R.Bottom:=Sz.CY
                     end;
                    case ChH of
                     'C':OffsetRgn(R2[C+1],X-(R.Right+R.Left) div 2,0);
                     'L':OffsetRgn(R2[C+1],X-R.Left,0);
                     'R':OffsetRgn(R2[C+1],X-R.Right,0)
                    else
                     raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Syntax error')
                    end;
                    case ChV of
                     'B':OffsetRgn(R2[C+1],0,Y-R.Bottom);
                     'C':OffsetRgn(R2[C+1],0,Y-(R.Top+R.Bottom) div 2);
                     'T':OffsetRgn(R2[C+1],0,Y-R.Top)
                    else
                     raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Syntax error')
                    end
                   end
                end;
           'U':begin
                CN:=0;
                for I:=1 to Length(S) do
                 if S[I]=',' then
                  Inc(CN);
                if CN<>5 then
                 raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Invalid number of parameters');
                for I:=1 to 5 do
                 begin
                  P:=Pos(',',S);
                  RR[I]:=GetValue(Copy(S,1,P-1));
                  Delete(S,1,P)
                 end;
                RR[6]:=GetValue(S);
                if Ch='U' then
                 R2[C+1]:=CreateRoundRectRgn(RR[1],RR[2],RR[3],RR[4],RR[5],RR[6])
                else
                 with Form do
                  R2[C+1]:=CreateRoundRectRgn(RR[1],RR[2],Width+RR[3],Height+RR[4],RR[5],RR[6])
               end;
           'X':Comb[C]:=Rgn_XOr;
           '{':if C<15 then
                begin
                 Inc(C);
                 Comb[C]:=Rgn_OR;
                 First[C]:=True
                end
               else
                raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Too many brackets');
           '}':if C>1 then
                begin
                 R2[C+1]:=0;
                 Dec(C)
                end
               else
                raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': ''}'' without ''{''')
          else
           raise ECannotTranslate.Create(SelfName+', line'+IntToStr(N)+': Unknown command');
          end;
          if R2[C+1]<>0 then
           if First[C] then
            begin
             R2[C]:=R2[C+1];
             First[C]:=False
            end
           else
            begin
             CombineRgn(R2[C],R2[C],R2[C+1],Comb[C]);
             DeleteObject(R2[C+1])
            end
         end;

       begin
        Result:=0;
        if Region='' then
         Exit;
        NameList.Add(SelfName);
        Comb[1]:=Rgn_OR;
        for N:=1 to 16 do
         R2[N]:=0;
        C:=1;
        S:=UpperCase(Region);
        N:=1;
        First[1]:=True;
        try
         P:=Pos(#13#10,S);
         if P<>0 then
          begin
           while P<>0 do
            begin
             ProcessString(Copy(S,1,P-1));
             Delete(S,1,P+1);
             P:=Pos(#13#10,S);
             Inc(N)
            end;
           ProcessString(S)
          end
         else
          ProcessString(S);
         Result:=R2[1];
         NameList.Delete(NameList.Count-1)
        except
         on E:Exception do
          begin
           for P:=1 to 16 do
            if R2[P]<>0 then
             DeleteObject(R2[P]);
           NameList.Clear;
           if (E is EExtRgnCrossLink) or (E is ECannotTranslate) then
            raise
          end
        end
       end;

    function TFRCoordinator.GetXOrigin;
     begin
      if csDesigning in Form.ComponentState then
       case Form.BorderStyle of
        bsDialog:Result:=SzFrame.X-DFrame.X;
        bsSingle,bsToolWindow:Result:=SzFrame.X-SFrame.X;
        bsNone:Result:=SzFrame.X;
        bsSizeable,bsSizeToolWin:Result:=0
       end
      else
       Result:=0
     end;

    function TFRCoordinator.GetYOrigin;
     begin
      if csDesigning in Form.ComponentState then
       case Form.BorderStyle of
        bsDialog:Result:=SzFrame.Y-DFrame.Y;
        bsSingle:Result:=SzFrame.Y-SFrame.Y;
        bsNone:Result:=SzFrame.Y+NCapt;
        bsSizeable:Result:=0;
        bsToolWindow:Result:=SzFrame.Y-SFrame.Y+NCapt-SmCapt;
        bsSizeToolWin:Result:=NCapt-SmCapt
       end
      else
       Result:=0
     end;

    function TFRCoordinator.DesignWidth;
     begin
      Result:=2*GetXOrigin
     end;

    function TFRCoordinator.DesignHeight;
     begin
      if csDesigning in Form.ComponentState then
       case Form.BorderStyle of
        bsDialog:Result:=2*(SzFrame.Y-DFrame.Y);
        bsSingle:Result:=2*(SzFrame.Y-SFrame.Y);
        bsNone:Result:=2*SzFrame.Y+NCapt;
        bsSizeable:Result:=0;
        bsToolWindow:Result:=2*(SzFrame.Y-SFrame.Y)+NCapt-SmCapt;
        bsSizeToolWin:Result:=NCapt-SmCapt
       end
      else
       Result:=0
     end;

    procedure TFRCoordinator.CallDefault;
     begin
      Message.Result:=CallWindowProc(OldWndProc,Form.Handle,Message.Msg,Message.wParam,Message.lParam)
     end;

    procedure Register;
     begin
      RegisterComponents('FormRgn',[TFormRegion,TCaptionRegion,TGlassRegion,TTextExtRgn,TPictExtRgn,TMacrosExtRgn,TMenuHotSpot,TSystemButton,TCaptionText,TFRManager]);
      RegisterPropertyEditor(TypeInfo(string),TRegionComp,'Region',TRegionProperty);
      RegisterPropertyEditor(TypeInfo(string),TMacrosExtRgn,'Region',TRegionProperty)
     end;

 initialization
  begin
   DFrame.X:=GetSystemMetrics(SM_CXDlgFrame);
   DFrame.Y:=GetSystemMetrics(SM_CYDlgFrame);
   SFrame.X:=GetSystemMetrics(SM_CXFixedFrame);
   SFrame.Y:=GetSystemMetrics(SM_CYFixedFrame);
   SzFrame.X:=GetSystemMetrics(SM_CXSizeFrame);
   SzFrame.Y:=GetSystemMetrics(SM_CYSizeFrame);
   NCapt:=GetSystemMetrics(SM_CYCaption);
   SmCapt:=GetSystemMetrics(SM_CYSmCaption);
   CHead:=nil
  end;

 finalization
  while Assigned(CHead) do
   CHead.Free;

 end.
